@@ -42,8 +42,12 @@
 *   V2.12 30-Jan-06, M Liu      Added FreeBSD port.
 *   V2.13 09-Jun-06, M Liu      Added HP-UX and AIX for LFS.
 *   V2.14 09-Jul-06, M Liu      Added MacOSX on i386 PC (macosXintel).
-*   V2.14 18-Apr-08, M Liu      Further modified MacOS X into macosXppc and 
-*                               maxosXintel
+*   V2.15 18-Apr-08, M Liu      Further modified MacOS X into macosXppc and 
+*                               maxosXintel. For PPC, pre-defined macro 
+*                               "__ppc__" and "__ppc64__" by gcc for 32/64-
+*                               bit mode, respectively on Mac.
+*   V2.16 28-Apr-09, M Liu      Added linux-PPC (pre-defined macro "__PPC__"
+*                               by gcc for either 32/64-bit).
 *
 ******************************************************************************/
 
@@ -63,6 +67,9 @@
 #      define MSVC67 
 #    endif
 #  endif
+#  if defined (_MSC_VER)
+#    define snprintf _snprintf
+#  endif
 #endif
 
 #if defined(MSDOS)              /* Microsoft C. */
@@ -114,7 +121,7 @@
 
 #if defined(linux) || defined(__CYGWIN__) || defined(__MINGW32__) || \
     defined(__FreeBSD__)
-#  if defined(PPC)
+#  if defined(PPC) || defined(__PPC__) 
 #    define POWERPC
 #  else
 #    if defined(i386) || defined(__amd64) || defined(__x86_64__) || \
@@ -224,13 +231,7 @@
 #  undef win32
 #  define unix
 #endif
-/*
-#if defined(__MACH__) && defined(i386)
-#  define unix
-#  define IBMPC
-#  define i386mac
-#endif
-*/
+
 /******************************************************************************
 * Determine variable argument list type - `stdarg' or `varargs'.  Currently,
 * only SunOS/bsd doesn't have `stdarg.h' (but SunOS/gnu and all Solaris do).
@@ -313,6 +314,9 @@
 #  if !defined(SALFORDC)
 #    include <dos.h>
 #  endif
+#  if defined(__MINGW32__)
+#    include <dirent.h>
+#  endif
 #  include <conio.h>
 #  include <sys\types.h>
 #  include <sys\stat.h>
@@ -398,18 +402,21 @@ typedef unsigned long uLong;
 
 #if defined(alphaosf) || defined(IRIX64bit) || defined(SOLARIS64) || \
     defined(__amd64) || defined(__x86_64__) || defined(__ia64__) || \
-    defined(AIX64) || defined(HP64) || defined(MACOSX64)
+    defined(AIX64) || defined(HP64) || defined(__PPC64__) || \
+    defined(__ppc64__)
    /***************************************************************************
    * `long' is 64 bits on a DEC Alpha/OSF1, an SGi/IRIX 6.x using the `-64'
    * option (64-bit objects) and a Solaris sparcv9, AMD64 and EM64T/IA64.
-   * They are also 64-bits on AIX and HP-UX if 64-bit mode is on.
+   * They are also 64-bits on AIX, HP-UX and PPC if 64-bit mode is on.
    * `int' is 32 bits in these cases.
    ***************************************************************************/
    typedef int Int32;
    typedef uInt uInt32;
+   typedef long Int64;
 #else
    typedef long Int32;
    typedef uLong uInt32;
+   typedef long long Int64;
 #endif
 
 typedef short Int16;
@@ -489,7 +496,8 @@ typedef sChar sByte;
 #    endif
 #    if defined(__osf__) || defined(__CYGWIN__) || \
         defined(vms) || defined(sgi) || defined(__MINGW32__) || \
-        defined(__FreeBSD__) || defined(macosXintel) || defined(macosXppc)
+        defined(__FreeBSD__) || defined(macosXintel) || defined(macosXppc) || \
+        defined(__PPC__)
 #      define FOPEN64 fopen
 #    else
 #      define FOPEN64 fopen64
@@ -539,7 +547,7 @@ typedef sChar sByte;
 
 /*****************************************************************************
 * Floating-point encodings.
-*   1..........Sun, SGi, IBM-RS, HP, NeXT, Macintosh
+*   1..........Sun, SGi, IBM-RS, HP, NeXT, Macintosh, PowerPC
 *   2..........DECstation, IBM-PC, Alpha (OSF/1), Alpha (OpenVMS - IEEE_FLOAT)
 *   3..........VAX, Alpha (OpenVMS - D_FLOAT)
 *   4..........Alpha (OpenVMS - G_FLOAT)
@@ -588,7 +596,7 @@ ap = _apT_; \
 #define NO_RECORD		(-1)
 #define NO_ENTRY		(-1)
 
-#define DU_MAX_DIR_LEN		512
+#define DU_MAX_DIR_LEN		256
 #define DU_MAX_NAME_LEN		256
 #define DU_MAX_PATH_LEN		DU_MAX_DIR_LEN + DU_MAX_NAME_LEN
 
@@ -769,12 +777,19 @@ ap = _apT_; \
 #define EPOCHdataType(dataType) (dataType == CDF_EPOCH)
 
 /******************************************************************************
- * EPOCH16dataType.
- *    Macro to check if an EPOCH16 data type.
- ******************************************************************************/
+* EPOCH16dataType.
+*    Macro to check if an EPOCH16 data type.
+******************************************************************************/
 
 #define EPOCH16dataType(dataType) (dataType == CDF_EPOCH16)
 
+/******************************************************************************
+* TT2000dataType.
+*    Macro to check if an CDF_TIME_TT2000 data type.
+******************************************************************************/
+
+#define TT2000dataType(dataType) (dataType == CDF_TIME_TT2000)
+
 /******************************************************************************
 * EofS (End of String).
 *   Macro which expands to a pointer to the NUL character at the end of a
@@ -997,6 +1012,8 @@ VISIBLE_PREFIX OFF_T MinLongLong PROTOARGs((OFF_T a, OFF_T b));
 VISIBLE_PREFIX int MaxInt PROTOARGs((int a, int b));
 VISIBLE_PREFIX Int32 MaxInt32 PROTOARGs((Int32 a, Int32 b));
 VISIBLE_PREFIX long MaxLong PROTOARGs((long a, long b));
+VISIBLE_PREFIX Int64 MaxInt64 PROTOARGs((Int64 a, Int64 b));
+VISIBLE_PREFIX Int64 MinInt64 PROTOARGs((Int64 a, Int64 b));
 VISIBLE_PREFIX OFF_T MaxLongLong PROTOARGs((OFF_T a, OFF_T b));
 VISIBLE_PREFIX int EndsWith PROTOARGs((char *s1, char *s2));
 VISIBLE_PREFIX int EndsWithIgCase PROTOARGs((char *s1, char *s2));
