@@ -24,6 +24,8 @@
 *   V1.3a 21-Feb-97, J Love	Removed RICE.
 *   V1.3b 28-Feb-97, J Love	Windows NT for MS Visual C/C++ on an IBM PC.
 *   V1.3c 14-Nov-97, J Love	More Windows NT.
+*   V3.3  10-Jan-09, M Liu      Added Mac or Linux on PPC running 64-bit mode.
+*   V3.3a 10-Dec-10, M Liu      Added a new epoch style for ISO 8601.
 *
 *****************************************************************************/
 
@@ -42,6 +44,7 @@
 
 #define INPUT_MAJOR             0
 #define useDEFAULTcacheSIZE	0
+#define useDEFAULTrecordNUM     0
 
 /*****************************************************************************
 * Macintosh-specific...
@@ -92,7 +95,7 @@
 #if defined(mac)
 #  define CHECKforABORTso		CheckForAbortSOmac();
 #else
-#  if defined(win32) && !defined(ALONE)
+#  if (defined(win32) && !defined(__MINGW32__)) && !defined(ALONE)
 #    define CHECKforABORTso		CheckForAbortSOwin32();
 #  else
 #    define CHECKforABORTso \
@@ -119,8 +122,12 @@
 * Directory constants.
 *****************************************************************************/
 
-#define DU_MAX_DIR_LEN		512
+#ifndef DU_MAX_DIR_LEN 
+#define DU_MAX_DIR_LEN		256
+#endif
+#ifndef DU_MAX_NAME_LEN 
 #define DU_MAX_NAME_LEN		256
+#endif
 #define DU_MAX_PATTERNS		16
 
 /*****************************************************************************
@@ -133,16 +140,26 @@
 #define EPOCH1_STYLE		1
 #define EPOCH2_STYLE		2
 #define EPOCH3_STYLE		3
+#define EPOCH4_STYLE		6
 #define EPOCHf_STYLE		4		/* Use C/Fortran format. */
 #define EPOCHx_STYLE		5		/* Use custom format. */
 
+/*****************************************************************************
+* TT2000/time constants.
+*****************************************************************************/
+
+#define TT2000_0_STYLE		0               /* Standard */
+#define TT2000_1_STYLE		1
+#define TT2000_2_STYLE		2
+#define TT2000_3_STYLE		3
+
 /*****************************************************************************
 * QOP constants.
 *****************************************************************************/
 
 #define QOP_MAX_ARGVs	40
 #define QOP_MAX_PARMs	40
-#define QOP_MAX_QUALs	30
+#define QOP_MAX_QUALs	35
 
 /*****************************************************************************
 * QOP structure.
@@ -168,11 +185,13 @@ char *qualOpt[QOP_MAX_QUALs];
 ******************************************************************************/
 
 #if defined(alphaosf) || defined(IRIX64bit) || defined(SOLARIS64) || \
-    defined(__amd64) || defined(__x86_64__) || defined(__ia64__)
+    defined(__amd64) || defined(__x86_64__) || defined(__ia64__) || \
+    defined(__PPC64__) || defined(__ppc64__)
 #  define Int32FORMAT	"%d"
 #  define Int64FORMAT   "%lld"
 #  define Int32uFORMAT	"%u"
 #  define Int32FORMATmod	""
+#  define Int64FORMATmod        "ll"
 #  define Int32hexFORMAT	"0x%08X"
 #  if defined(alphaosf)
 #    define Int64hexFORMAT      "0x%016X"
@@ -181,15 +200,16 @@ char *qualOpt[QOP_MAX_QUALs];
 #  endif
 #else
 #  define Int32FORMAT	"%ld"
-#if defined(win32)
+#if defined(win32) || defined(__MINGW32__)
 #  define Int64FORMAT   "%I64d"
 #else
 #  define Int64FORMAT   "%lld"
 #endif
 #  define Int32uFORMAT	"%lu"
 #  define Int32FORMATmod	"l"
+#  define Int64FORMATmod	"ll"
 #  define Int32hexFORMAT	"0x%08lX"
-#if defined(win32)
+#if defined(win32) || defined(__MINGW32__)
 #  define Int64hexFORMAT        "0x%016I64X"
 #else
 #  define Int64hexFORMAT        "0x%016llX"
@@ -309,7 +329,8 @@ struct HyperStruct {
 #define MAX_SEVERITY_TEXT_LEN		20
 #define MAX_MESSAGE_TEXT_LEN		80
 #define MAX_PROGRAM_NAME_LEN		25
-#define MAX_ENCODED_INT4_LEN		10
+#define MAX_ENCODED_INT4_LEN		11
+#define MAX_ENCODED_INT8_LEN		20
 #define MAX_REPORTMASK_LEN		40
 #define MAX_oTEXT_LEN			150
 #define MAX_CACHESIZES_LEN		30
@@ -630,12 +651,15 @@ Logical GTx PROTOARGs((void *, void *, long, long));
 Logical IsDir PROTOARGs((char *));
 Logical IsWild PROTOARGs((char *));
 Logical IsCDF PROTOARGs((char *));
-void EncodeRecordJustify PROTOARGs((char *string, long recN, int minWidth));
+void EncodeRecordJustify PROTOARGs((char *string, long recN, int minWidth,
+  size_t width
+));
 void EncodeIndicesJustify PROTOARGs((
-  char *string, long numDims, long indices[], int minWidth
+  char *string, long numDims, long indices[], int minWidth, size_t width
 ));
 void EncodeRecordIndicesJustify PROTOARGs((
-  char *string, long recN, long numDims, long indices[], int minWidth
+  char *string, long recN, long numDims, long indices[], int minWidth,
+  size_t width
 ));
 void Justify PROTOARGs((char *string, int minWidth));
 void RemoveLeadingBlanks PROTOARGs((char *string));
@@ -656,21 +680,23 @@ void PageOLH PROTOARGs((char *, char *));
 int WriteOut PROTOARGs((FILE *, char *));
 char PickDelimiter PROTOARGs((char *, size_t));
 void ConvertDataType PROTOARGs((long, long, void *, long, long, void *));
-int EncodeDimensionality PROTOARGs((char *, long, long *));
+int EncodeDimensionality PROTOARGs((char *, long, long *, size_t));
 int EncodeVariances PROTOARGs((char *, long, long, long *));
-int EncodeRecordIndices PROTOARGs((char *, long, long, long, long *, long *));
-int EncodeRecordIndicesVARY PROTOARGs((char *, long, long, long *));
+int EncodeRecordIndices PROTOARGs((char *, long, long, long, long *, long *,
+  size_t
+));
+int EncodeRecordIndicesVARY PROTOARGs((char *, long, long, long *, size_t));
 int EncodeString PROTOARGs((long, char *, char *, int, int));
-int EncodeValue PROTOARGs((long, void *, char *, int));
+int EncodeValue PROTOARGs((long, void *, char *, int, size_t));
 int EncodeValueFormat PROTOARGs((
   long dataType, void *binary, char *text, char *format, int minWidth,
-  int maxWidth, int style
+  int maxWidth, int style, size_t width
 ));
 int EncodeValuesFormat PROTOARGs((
   long dataType, long numElems, void *binary, char *text, char *format,
-  int minWidth, int maxWidth, int style
+  int minWidth, int maxWidth, int style, size_t width
 ));
-void EncodeNegativeZero PROTOARGs((char *string, char *format));
+void EncodeNegativeZero PROTOARGs((char *string, char *format, size_t width));
 void WriteStringValue PROTOARGs((FILE *, long, void *, int, int));
 void WriteEntryValue PROTOARGs((FILE *, long, long, void *, int, int));
 char *EncodingToken PROTOARGs((long));
@@ -779,6 +805,10 @@ void InitHyperParms PROTOARGs((
   struct HyperStruct *hyperX, struct GroupStruct *groups, long numDims,
   long *nHypers, long *nValues
 ));
+void InitHyperParmsFromAnywhere PROTOARGs((
+  struct HyperStruct *hyperX, struct GroupStruct *groups, long numDims,
+  long *nHypers, long *nValues, long strRec
+));
 void IncrHyperParms PROTOARGs((
   struct HyperStruct *hyperX, struct GroupStruct *groups, long numDims,
   Logical rowMajor, long *nValues
@@ -802,16 +832,21 @@ Logical SameSparsenesses PROTOARGs((
   long sRecordsType1, long sArraysType1, long sArraysParms1[CDF_MAX_PARMS],
   long sRecordsType2, long sArraysType2, long sArraysParms2[CDF_MAX_PARMS]
 ));
-int LongWidth PROTOARGs((long value));
+int IntWidth PROTOARGs((int value));
 int Long64Width PROTOARGs((OFF_T value));
 void DisplayIdentification PROTOARGs((char *name));
 Logical  ParseCacheSizes PROTOARGs((
   char *sizes, long *workingCache, long *stageCache, long *compressCache
 ));
+Logical  ParseRecordRange PROTOARGs((
+  char *sizes, long *startingRec, long *endingRec
+));
 int FindUniqueMatch PROTOARGs((char *target, char *strings[]));
 void ParseStringForTokens PROTOARGs((int token, char *string, char *items[]));
 int GetNumTokens PROTOARGs((int token, char *string));
-
+Logical ValidateQual PROTOARGs((
+  char *qual, char *validQuals[]
+));
 #if defined(mac)
 void MacExecuteSO PROTOARGs((
   Logical (*exeFnc)(int argC, char *argV[]),
