@@ -16,6 +16,8 @@
 *                               on-screen width.
 *   V1.2d 17-Nov-97, J Love	Windows NT/Visual C++.
 *   V1.2e 14-Dec-97, J Love	Added ALPHAVMSi encoding.
+*   V3.3  10-Apr-09, M Liu      Added Linux port on PPC.
+*   V3.3a 10-Dec-10, M Liu      Added a new epoch style for ISO 8601.
 *
 ******************************************************************************/
 
@@ -56,6 +58,12 @@
 #define SETTINGSqual            15
 #define BATCHqual               16
 #define HELPqual                17
+#define INCLUDEqual             18
+#define EXCLUDEqual             19
+#define TIMEFIRSTSEQUENCEqual   20
+#define SETTINGSSEQUENCEqual    21
+#define EPOCHRANGEqual          22
+#define RECORDRANGEqual         23
 
 #define FORMATopt               0
 #define NOFORMATopt             1
@@ -99,6 +107,8 @@
 #define NOPREALLOCATEopt        39
 #define HEADINGopt              40
 #define NOHEADINGopt            41
+#define ISO8601opt              42
+#define POUNDEDHEADINGopt       43
 
 /******************************************************************************
 * Miscellaneous.
@@ -241,12 +251,15 @@
 #define RESTOREkey_SIMPLE               KB_CTRL_L
 
 #if defined(alphaosf) || defined(IRIX64bit) || defined(SOLARIS64) || \
-    defined(__amd64) || defined(__x86_64__) || defined(__ia64__)
+    defined(__amd64) || defined(__x86_64__) || defined(__ia64__) || \
+    defined(__PPC64__) || defined(__ppc64__)
 #  define Int32FORMATstandard     "%11d"
 #  define Int32uFORMATstandard    "%10d"
+#  define Int64FORMATstandard     "%20lld"
 #else
 #  define Int32FORMATstandard     "%11ld"
 #  define Int32uFORMATstandard    "%10ld"
+#  define Int64FORMATstandard     "%20lld"
 #endif
 
 #define EPOCHx_FORMAT_STANDARD  "*Unsupported*"
@@ -442,6 +455,7 @@
 #define OUTPUTtoFILEh           3
 #define OUTPUTtoFILEv           4
 #define OUTPUTtoCDF             5
+#define OUTPUTtoFILEcdaweb      6
 
 /******************************************************************************
 * Monotonicities.
@@ -741,16 +755,17 @@ char dots[] = "...";
 char *monos[] = { "n/a", "Unknown", "Increase", "Decrease", "False" };
 char *epochStyles[] = {
   "standard", "alternate/1", "alternate/2", "alternate/3",
-  "Format/C_Fortran", "Format/custom"
+  "Format/C_Fortran", "Format/custom", "ISO8601"
 };
 char *encodings[] = {
   NULL, "NETWORK", "SUN", "VAX", "DECSTATION", "SGi", "IBMPC", "IBMRS",
-  "HOST", "MAC", NULL, "HP", "NeXT", "ALPHAOSF1", "ALPHAVMSd", "ALPHAVMSg",
-  "ALPHAVMSi", "PowerPC",
+  "HOST", "PPC", NULL, "HP", "NeXT", "ALPHAOSF1", "ALPHAVMSd", "ALPHAVMSg",
+  "ALPHAVMSi", "PowerPC"
 };
 char *majorities[] = { "input", "row", "column" };
 char *compressions[] = { "no", "rle", "huff", "ahuff", "gzip" };
 char *checksums[] = {"no", "md5"};
+char *headings[] = { "no", "YES", "POUNDEDHEADING" };
 char readingCDF[] = "reading input CDF";
 char writingCDF[] = "writing output CDF";
 long dimIndices_0[CDF_MAX_DIMS] = { 0,0,0,0,0,0,0,0,0,0 };
@@ -767,6 +782,28 @@ char settingsFile[DU_MAX_PATH_LEN+1] = "export.set";
 #endif
 char outputText[DU_MAX_PATH_LEN+1] = "default.lis";
 char outputCDF[CDF_PATHNAME_LEN+1] = "default";
+long includeVars;
+char varsList[50*(CDF_VAR_NAME_LEN256+1)];
+long outputVars;
+Logical cdaweb;
+int  poundedheading;
+char **vars;
+int  numVars;
+int  numVarsO;
+long range;
+double epochStart, epochEnd;
+double epoch16Start[2], epoch16End[2];
+long long tt2000Start, tt2000End;
+long recordStart, recordEnd;
+int numEpochs;
+char **epochs;
+int *epochIndx;
+long *epochMaxs;
+int *epochTypes;
+int numGroups;
+char ***varsInGroup;
+struct ItemStruct ***varPtrsInGroup;
+int *numVarsInGroup;
 #else
 extern int batchMode;
 extern Logical simpleMode;
@@ -801,6 +838,28 @@ extern long CDFchecksum;
 extern char settingsFile[DU_MAX_PATH_LEN+1];
 extern char outputText[DU_MAX_PATH_LEN+1];
 extern char outputCDF[CDF_PATHNAME_LEN+1];
+extern long includeVars;
+extern long outputVars;
+extern Logical cdaweb;
+extern int  poundedheading;
+extern char varsList[50*(CDF_VAR_NAME_LEN256+1)];
+extern char **vars;
+extern int  numVars;
+extern int  numVarsO;
+extern Logical range;
+extern double epochStart, epochEnd;
+extern double epoch16Start[2], epoch16End[2];
+extern long long tt2000Start, tt2000End;
+extern long recordStart, recordEnd;
+extern int numEpochs;
+extern char **epochs;
+extern int *epochIndx;
+extern long *epochMaxs;
+extern int *epochTypes;
+extern int numGroups;
+extern char ***varsInGroup;
+extern struct ItemStruct ***varPtrsInGroup;
+extern int *numVarsInGroup;
 #endif
 
 /******************************************************************************
@@ -859,16 +918,22 @@ void RestoreSettings PROTOARGs((void));
 Logical ToWalk PROTOARGs((void));
 Logical ToScreenHori PROTOARGs((void));
 Logical ToFileHori PROTOARGs((void));
+Logical ToFileCDAweb PROTOARGs((void));
 CDFstatus EncodeLineHori PROTOARGs((
   char *line, long recN, int *filterStatus, struct ItemStruct *exportHead,
-  Logical standard
+  Logical standard, size_t width
+));
+CDFstatus EncodeLineHoriN PROTOARGs((
+  char *line, long recN, int *filterStatus, int ix, int ia, int **flags,
+  Logical standard, size_t width
 ));
 Logical ToScreenVert PROTOARGs((void));
 Logical ToFileVert PROTOARGs((void));
 CDFstatus EncodeLineVert PROTOARGs((
   char *line, long recN, long valueN, long numDims,
   long indices[CDF_MAX_DIMS], Logical same, int *filterStatus,
-  struct ItemStruct *exportHead, Logical outRowMajor, Logical standard
+  struct ItemStruct *exportHead, Logical outRowMajor, Logical standard,
+  size_t width
 ));
 Logical ListAttributes PROTOARGs((FILE *oFp, Logical *cdfFatal));
 Logical ToCDF PROTOARGs((CDFid inID));
@@ -935,6 +1000,8 @@ void BuildExportList PROTOARGs((
   struct ItemStruct **exportHead, Logical walking
 ));
 void RemoveExportItems PROTOARGs((struct ItemStruct **exportHead));
+void ReorderExportItems PROTOARGs((struct ItemStruct **exportHead));
+void ReorderGroupItems PROTOARGs((struct ItemStruct **exportHead));
 void FreeExportBuffers PROTOARGs((struct ItemStruct *exportHead));
 void FilterHypers PROTOARGs((struct ItemStruct *hyperHead, long nValues));
 long FindFirstRecord PROTOARGs((
@@ -955,7 +1022,17 @@ int PreAllocateRecords PROTOARGs((
   struct ItemStruct *hyperHead, long *firstRec, long *lastRec
 ));
 void OptDefaults PROTOARGs((void));
-
+Logical ParseEpochRange PROTOARGs((char **epochRanges));
+Logical LoadVarsTextFile PROTOARGs((char *varsFile));
+Logical IsaVirtualVariable PROTOARGs((Logical zVAR, char *varName));
+CDFstatus GetVirtualVariableFunc PROTOARGs((Logical zVAR, char *varName, 
+char *funcName));
+CDFstatus ReadRealorVirtualVariableEpoch PROTOARGs((Logical zVAR,
+char *varName, int ind, double **epochValues));
+CDFstatus ReadVirtualVariableApplyQaflag PROTOARGs((Logical zVar, char *varName,
+int *qaFlags));
+int FindUsedEpochs PROTOARGs((struct ItemStruct **itemHead, int *index));
+CDFstatus LoadVirtualEpochs PROTOARGs((double **epochValues));
 #if defined(mac)
 Logical ExportQOPs PROTOARGs((
   int *argC, char **argV[]
