@@ -304,7 +304,8 @@ switch (Va->item) {
     * Validate arguments.
     **************************************************************************/
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
-    if (!ValidDataType(dataType)) return BAD_DATA_TYPE;
+    if (!ValidDataType(dataType) || InValidDataType(dataType))
+      return BAD_DATA_TYPE;
     if (numElements < 1) return BAD_NUM_ELEMS;
     if (!STRINGdataType(dataType) && numElements != 1) return BAD_NUM_ELEMS;
     if (zOp) {
