@@ -19,6 +19,13 @@
 *   V2.0  08-Apr-04, M Liu      Reset the current variable offset when that
 *                               variable is deleted.
 *   V2.1  29-Jun-04, M Liu      Added LFS (Large File Support > 2G).
+*   V2.2  08-Jan-09, M Liu      Update the attrnum in AEDR when an attribute
+*                               is deleted.
+*   V2.3  08-May-10, M Liu      Reset firstRecInVVR/lastRecInVVR when a 
+*                               record(s) gets deleted.
+*   V2.4  24-Jan-11, M Liu      Modified UpdateIndexEntries_r_64 to acquire
+*                               physical record size to set VXR's last record.
+*   V2.5  09-May-12, M Liu      Remodified UpdateIndexEntries_r_64.
 *
 ******************************************************************************/
 
@@ -103,6 +110,7 @@ switch (Va->item) {
     }
     FreeCDFid (CDF, FALSE);
     Cur->cdf = NULL;
+    ReduceOpenCDFsCount();
     break;
   }
   /****************************************************************************
@@ -445,6 +453,7 @@ switch (Va->item) {
     struct GDRstruct64 GDR;
     struct ADRstruct64 ADR, ADRt;
     struct AEDRstruct64 AEDRt;
+    OFF_T eOffset, eNext;
     /**************************************************************************
     * Get pointer to current CDF and locate the current attribute.
     **************************************************************************/
@@ -524,6 +533,38 @@ switch (Va->item) {
 	AbortAccess64 (CDF, UPDATE, noDELETE);
 	return pStatus;
       }
+      eOffset = ADRt.AgrEDRhead;
+      while (eOffset != 0) {
+        if (!sX(ReadAEDR64(CDF->fp,eOffset,
+                           AEDR_AEDRNEXT,&eNext,
+                           AEDR_NULL),&pStatus)) {
+          AbortAccess64 (CDF, UPDATE, noDELETE);
+          return pStatus;
+        }
+        if (!sX(WriteAEDR64(CDF,CDF->fp,eOffset,
+                            AEDR_ATTRNUM,&ADRt.Num,
+                            AEDR_NULL),&pStatus)) {
+          AbortAccess64 (CDF, UPDATE, noDELETE);
+          return pStatus;
+        }
+        eOffset = eNext;
+      }
+      eOffset = ADRt.AzEDRhead;
+      while (eOffset != 0) {
+        if (!sX(ReadAEDR64(CDF->fp,eOffset,
+                           AEDR_AEDRNEXT,&eNext,
+                           AEDR_NULL),&pStatus)) {
+          AbortAccess64 (CDF, UPDATE, noDELETE);
+          return pStatus;
+        }
+        if (!sX(WriteAEDR64(CDF,CDF->fp,eOffset,
+                            AEDR_ATTRNUM,&ADRt.Num,
+                            AEDR_NULL),&pStatus)) {
+          AbortAccess64 (CDF, UPDATE, noDELETE);
+          return pStatus;
+        }
+        eOffset = eNext;
+      }
       tOffset = ADRt.ADRnext;
     }
     /**************************************************************************
@@ -696,7 +737,7 @@ Int32 lastRec;
     * Standard variable in a single-file CDF.
     **************************************************************************/
     case STANDARD_: {
-      Int32 aboveRecord, deleteCount;
+      Int32 aboveRecord, deleteCount; Int32 adjust;
       OFF_T vxrHead;
       /************************************************************************
       * Verify/adjust the records to be deleted.
@@ -718,9 +759,9 @@ Int32 lastRec;
       if (!sX(ReadVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
 		        VDR_VXRHEAD,&vxrHead,
 		        VDR_NULL),&pStatus)) return pStatus;
-      if (!sX(UpdateIndexEntries_r_64(CDF->fp,vxrHead,
-				      aboveRecord,
-				      deleteCount),&pStatus)) return pStatus;
+      if (!sX(UpdateIndexEntries_r_64(CDF->fp,vxrHead,aboveRecord,deleteCount),
+				      &pStatus)) 
+	return pStatus;
       /************************************************************************
       * Update the control information.
       ************************************************************************/
@@ -736,6 +777,9 @@ Int32 lastRec;
 	else
 	  Var->maxAllocated -= deleteCount;
       }
+      Var->firstRecInVVR = NO_RECORD;
+      Var->lastRecInVVR = NO_RECORD;
+      Var->offsetOfVVR64 = (OFF_T) NO_OFFSET64;
       /************************************************************************
       * Update the maximum record field in the VDR.
       ************************************************************************/
@@ -747,7 +791,7 @@ Int32 lastRec;
 	if (!sX(WriteVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
 			   VDR_MAXREC,&(Var->maxRec),
 			   VDR_NULL),&pStatus)) return pStatus;
-      }
+      }	      
       break;
     }
     /**************************************************************************
@@ -800,12 +844,15 @@ Int32 lastRec;
 			   VDR_MAXREC,&(Var->maxRec),
 			   VDR_NULL),&pStatus)) return pStatus;
       }
+      Var->firstRecInVVR = NO_RECORD;
+      Var->lastRecInVVR = NO_RECORD;
+      Var->offsetOfVVR64 = (OFF_T) NO_OFFSET64;
       break;
     /**************************************************************************
     * Compressed records.
     **************************************************************************/
     case COMPRESSED_: {
-      Int32 aboveRecord, deleteCount;
+      Int32 aboveRecord, deleteCount; Int32 adjust;
       OFF_T vxrHead;
       /************************************************************************
       * Flush/clear staging area first.
@@ -814,6 +861,9 @@ Int32 lastRec;
       Var->stage.firstRec = NO_RECORD;
       Var->stage.lastRec = NO_RECORD;
       Var->stage.dotOffset64 = (OFF_T) NO_OFFSET64;
+      Var->firstRecInVVR = NO_RECORD;
+      Var->lastRecInVVR = NO_RECORD;
+      Var->offsetOfVVR64 = (OFF_T) NO_OFFSET64;
       /************************************************************************
       * Verify/adjust the records to be deleted.
       ************************************************************************/
@@ -834,9 +884,9 @@ Int32 lastRec;
       if (!sX(ReadVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
 		        VDR_VXRHEAD,&vxrHead,
 		        VDR_NULL),&pStatus)) return pStatus;
-      if (!sX(UpdateIndexEntries_r_64(CDF->fp,vxrHead,
-				      aboveRecord,
-				      deleteCount),&pStatus)) return pStatus;
+      if (!sX(UpdateIndexEntries_r_64(CDF->fp,vxrHead,aboveRecord,deleteCount),
+				      &pStatus))
+	return pStatus;
       /************************************************************************
       * Update the maximum record field in the VDR.
       ************************************************************************/
@@ -862,6 +912,9 @@ Int32 lastRec;
       Var->stage.firstRec = NO_RECORD;
       Var->stage.lastRec = NO_RECORD;
       Var->stage.dotOffset64 = (OFF_T) NO_OFFSET64;
+      Var->firstRecInVVR = NO_RECORD;
+      Var->lastRecInVVR = NO_RECORD;
+      Var->offsetOfVVR64 = (OFF_T) NO_OFFSET64;
       /************************************************************************
       * Verify/adjust the records to be deleted.
       ************************************************************************/
@@ -1271,7 +1324,7 @@ OFF_T irSize;
       Int32 nRecords1, nBytes1, nDeleteRecords;
       Int32 nDeleteBytes, firstRecord2, lastRecord2, nRecords2;
       Int32 nBytes2, nExtraBytes, vvr_ = VVR_;
-      Int32 nRemainingBytes;
+      OFF_T nRemainingBytes;
       OFF_T deleteOffset, offset2, vvrOffset, recordSize;
       /************************************************************************
       * A block of records will remain at the beginning and at the end.  First
@@ -1314,7 +1367,7 @@ OFF_T irSize;
 	/**********************************************************************
 	* If the remaining bytes can be changed to a UIR...
 	**********************************************************************/
-	nRemainingBytes = nDeleteBytes + nBytes2 + nExtraBytes;
+	nRemainingBytes = (OFF_T) nDeleteBytes + nBytes2 + nExtraBytes;
 	if (nRemainingBytes >= UIR_BASE_SIZE64) {
 	  recordSize = VVR_BASE_SIZE64 + nBytes1;
 	  if (!sX(WriteVVR64(CDF->fp,VXR->Offset[entryN],
@@ -1346,7 +1399,7 @@ OFF_T irSize;
 	* going to be used for the new VVR containing the second block of
 	* remaining records.
 	**********************************************************************/
-	nRemainingBytes = nDeleteBytes - VVR_BASE_SIZE64;
+	nRemainingBytes = (OFF_T) nDeleteBytes - VVR_BASE_SIZE64;
 	if (nRemainingBytes < UIR_BASE_SIZE64)
 	  recordSize = VVR_BASE_SIZE64 + nBytes1 + nRemainingBytes;
 	else {
@@ -1775,7 +1828,7 @@ OFF_T offset;
   * this VXR with the new VXR's offset, and write the new VXR.
   ****************************************************************************/
   if (VXR.VXRnext == (OFF_T) ZERO_OFFSET64) {
-    if (!sX(AllocateIR64(CDF,(Int32)VXR_BASE_SIZE64,
+    if (!sX(AllocateIR64(CDF,(OFF_T) VXR_BASE_SIZE64,
 		         &newVXRoffset),&pStatus)) return pStatus;
     VXR.VXRnext = newVXRoffset;
     if (!sX(WriteVXR64(CDF->fp,vxrOffset,
@@ -1824,10 +1877,9 @@ Int32 recordCount;
 			      &irType),&pStatus)) return pStatus;
 	 switch (irType) {
 	   case VXR_:
-	     if (!sX(UpdateIndexEntries_r_64(fp,VXR.Offset[entryN],
-					     aboveRecord,
-					     recordCount),&pStatus)) return
-								  pStatus;
+	     if (!sX(UpdateIndexEntries_r_64(fp,VXR.Offset[entryN],aboveRecord,
+					     recordCount),&pStatus))
+		return pStatus;
 	     break;
 	   case VVR_:
 	   case CVVR_:
@@ -1836,7 +1888,7 @@ Int32 recordCount;
 	     return CORRUPTED_V3_CDF;
 	 }
 	 if (VXR.First[entryN] > aboveRecord) VXR.First[entryN] -= recordCount;
-	 VXR.Last[entryN] -= recordCount;
+         VXR.Last[entryN] -= recordCount;
 	 modified = TRUE;
        }
     }
