@@ -805,8 +805,8 @@ switch (Va->item) {
   case rVAR_MAXallocREC_:
   case zVAR_MAXallocREC_: {
     Logical zOp = (Va->item == zVAR_MAXallocREC_), zVar;
-    struct CDFstruct *CDF;
-    Int32 maxRec, lastAllocatedRecN; OFF_T offset;
+    struct CDFstruct *CDF; struct VarStruct *Var;
+    Int32 maxRec, lastAllocatedRecN; OFF_T offset; int vType;
     long *maxAllocated = va_arg (Va->ap, long *);
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
@@ -815,7 +815,25 @@ switch (Va->item) {
     zVar = CurrentVarMode(CDF,zOp);
     if (zModeON(CDF) || zVar) offset = CDF->CURzVarOffset64;
     else offset = CDF->CURrVarOffset64;
+    if (zModeON(CDF))
+      if (CDF->CURzVarNum < CDF->NrVars)
+        Var = CDF->rVars[(int)CDF->CURzVarNum];
+      else
+        Var = CDF->zVars[(int)(CDF->CURzVarNum - CDF->NrVars)];
+    else
+      Var = BOO(zOp,CDF->zVars[(int)CDF->CURzVarNum],
+                    CDF->rVars[(int)CDF->CURrVarNum]);
 
+    if (!sX(ReadVDR64(CDF,CDF->fp,offset,zVar,
+                      VDR_MAXREC,&maxRec,
+                      VDR_NULL),&pStatus)) {
+      AbortAccess64 (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
+    if (!sX(VariableType64(CDF,offset,zVar,&vType),&pStatus)) {
+      AbortAccess64 (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }    
     /**************************************************************************
     * If a single-file CDF, pass back the maximum record number allocated.
     * If a multi-file CDF, pass back the maximum record number written (which
@@ -826,17 +844,31 @@ switch (Va->item) {
 	AbortAccess64 (CDF, UPDATE, noDELETE);
 	return pStatus;
       }
+      if (vType == SPARSE_RECORDS_) {
+          if (Var != NULL) {
+            if (Var->stage.areaOffset64 != (OFF_T) NO_OFFSET64 &&
+                Var->stage.firstRec != NO_RECORD) {
+              Int32 tmp = (Var->stage.lastRec > maxRec ? maxRec :
+                           Var->stage.lastRec);
+              lastAllocatedRecN += (tmp - Var->stage.firstRec + 1);
+            }
+          }
+      } else if (vType == COMPRESSED_ ||
+                 vType == SPARSE_COMPRESSED_RECORDS_) {
+        if (Var != NULL) {
+          if (Var->stage.areaOffset64 != (OFF_T) NO_OFFSET64 &&
+              Var->stage.firstRec != NO_RECORD &&
+              Var->stage.dotOffset64 == (OFF_T) NO_OFFSET64) {
+            Int32 tmp = (Var->stage.lastRec > maxRec ? maxRec :
+                         Var->stage.lastRec);
+            lastAllocatedRecN += (tmp - Var->stage.firstRec + 1);
+          }
+        }
+      }
       *maxAllocated = lastAllocatedRecN;
     }
-    else {
-      if (!sX(ReadVDR64(CDF,CDF->fp,offset,zVar,
-		        VDR_MAXREC,&maxRec,
-		        VDR_NULL),&pStatus)) {
-	AbortAccess64 (CDF, UPDATE, noDELETE);
-	return pStatus;
-      }
+    else
       *maxAllocated = (long) maxRec;
-    }
     break;
   }
   /****************************************************************************
@@ -931,6 +963,7 @@ switch (Va->item) {
   case zVAR_NUMallocRECS_: {
     Logical zOp = (Va->item == zVAR_NUMallocRECS_), zVar;
     struct CDFstruct *CDF; Int32 nAllocated; int vType; OFF_T offset;
+    Int32 tMaxRec; struct VarStruct *Var;
     long *nAllocatedP = va_arg (Va->ap, long *);
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
@@ -939,8 +972,17 @@ switch (Va->item) {
     zVar = CurrentVarMode(CDF,zOp);
     if (zModeON(CDF) || zVar) offset = CDF->CURzVarOffset64;
     else offset = CDF->CURrVarOffset64;
+    if (zModeON(CDF))
+      if (CDF->CURzVarNum < CDF->NrVars)
+        Var = CDF->rVars[(int)CDF->CURzVarNum];
+      else
+        Var = CDF->zVars[(int)(CDF->CURzVarNum - CDF->NrVars)];
+    else
+      Var = BOO(zOp,CDF->zVars[(int)CDF->CURzVarNum],
+                    CDF->rVars[(int)CDF->CURrVarNum]);
 
     if (!sX(ReadVDR64(CDF,CDF->fp,offset,zVar,
+		      VDR_MAXREC,&tMaxRec,
 		      VDR_NULL),&pStatus)) {
       AbortAccess64 (CDF, UPDATE, noDELETE);
       return pStatus;
@@ -958,13 +1000,34 @@ switch (Va->item) {
       case COMPRESSED_:
       case SPARSE_COMPRESSED_RECORDS_:
       case SPARSE_ARRAYS_:
-      case SPARSE_RECORDS_AND_ARRAYS_:
-
+      case SPARSE_RECORDS_AND_ARRAYS_: {
 	if (!sX(IndexingStatistics64(CDF,offset,
 				     zVar,NULL,NULL,
 				     &nAllocated,
 				     NULL,NULL),&pStatus)) return pStatus;
+        if (vType == SPARSE_RECORDS_) {
+          if (Var != NULL) {
+            if (Var->stage.areaOffset64 != (OFF_T) NO_OFFSET64 &&
+                Var->stage.firstRec != NO_RECORD) {
+              Int32 tmp = (Var->stage.lastRec > tMaxRec ? tMaxRec :
+                           Var->stage.lastRec);
+              nAllocated += (tmp - Var->stage.firstRec + 1);
+            }
+          }
+        } else if (vType == COMPRESSED_ ||
+                   vType == SPARSE_COMPRESSED_RECORDS_) {
+          if (Var != NULL) {
+            if (Var->stage.areaOffset64 != (OFF_T) NO_OFFSET64 &&
+                Var->stage.firstRec != NO_RECORD &&
+                Var->stage.dotOffset64 == (OFF_T) NO_OFFSET64) {
+              Int32 tmp = (Var->stage.lastRec > tMaxRec ? tMaxRec :
+                           Var->stage.lastRec);              
+              nAllocated += (tmp - Var->stage.firstRec + 1);
+            }
+          }
+        }
 	break;
+      }
       case IN_MULTI_: {
 	Int32 maxRec;
 	if (!sX(ReadVDR64(CDF,CDF->fp,offset,zVar,
