@@ -29,6 +29,10 @@
 *   V2.0  08/Apr-04, M liu      Replaced VSTREAM.STATS with VSTREAM_STATS.
 *   V2.1  13-Oct-06, M Liu      Changed to allow upper and lower case CDF  
 *                               name to be used on win32.
+*   V2.2  12-Aug-10, M Liu      Ensured the filled file path name not overrun
+*                               the defined string in BuildFilePath.
+*   V2.3  22-Aug-10, M Liu      Modified NulPad to use memset to speed up the
+*                               padding process.
 *
 ******************************************************************************/
 
@@ -788,7 +792,7 @@ char *pathBase;         /* Base pathname. */
 Logical noAppend;       /* Should extensions/version numbers be appended? */
 Logical upperCase;      /* Should uppercase extensions be appended? */
 Logical versionNumber;  /* Should a version number of `;1' be appended? */
-Int32 varN;              /* Variable number.  N/a if a `cdf' file. */
+Int32 varN;             /* Variable number.  N/a if a `cdf' file. */
 char pathX[DU_MAX_PATH_LEN+1];
 			/* The expanded path w/ extensions/version numbers. */
 {
@@ -800,11 +804,13 @@ char pathX[DU_MAX_PATH_LEN+1];
 	break;
       case Vt:
 	strcatX (pathX, (upperCase ? ".V" : ".v"), DU_MAX_PATH_LEN);
-	sprintf (EofS(pathX), "%d", varN);
+  	snprintf (EofS(pathX), (size_t) DU_MAX_PATH_LEN+1-strlen(pathX), 
+                  "%d", (int) varN);
 	break;
       case Zt:
 	strcatX (pathX, (upperCase ? ".Z" : ".z"), DU_MAX_PATH_LEN);
-	sprintf (EofS(pathX), "%d", varN);
+	snprintf (EofS(pathX), (size_t) DU_MAX_PATH_LEN+1-strlen(pathX),
+                  "%d", (int) varN);
 	break;
     }
     strcatX (pathX, (versionNumber ? ";1" : ""), DU_MAX_PATH_LEN);
@@ -823,8 +829,15 @@ char *string;
 int length;
 {
   int i;
-  for (i = strlen(string); i < length; i++) string[i] = NUL;
+  int j;
+/*
+  for (i = (int) strlen(string); i < length; i++) string[i] = NUL;
   string[length] = NUL;
+*/
+  j = (int) strlen(string);
+  i = length - j;
+  if (i > 0) 
+    memset(string+j, '\0', i+1);
   return;
 }
 
@@ -948,7 +961,8 @@ int firstByte;
 int lastByte;
 {
   int i;
-  for (i = firstByte; i <= lastByte; i++) ((Byte *)buffer)[i] = 0;
+  /* for (i = firstByte; i <= lastByte; i++) ((Byte *)buffer)[i] = 0; */
+  memset ((Byte *)buffer+firstByte, 0, (size_t) (lastByte - firstByte + 1));
   return;
 }
 
