@@ -21,9 +21,11 @@
 *                               in READONLYon mode.
 *   V3.2a 11-Apr-08, M Liu      Modified Read32s to eliminate the potential 
 *                               buffer overflow.
-*   V3.2b 15-Sep-08, M Liu      Modified ReadAEDRList64 to pass in the number of
+*   V3.3  28-Jul-08, M Liu      Modified to work with CDF file validation.
+*   V3.3a 15-Sep-08, M Liu      Modified ReadAEDRList to pass in the number of
 *                               entries to set up the list, not just the last
-*                               entry number.  
+*                               entry number.
+*   V3.4  12-Aug-10, M Liu      Replaced strcpy by strcpyX in ReadVDR. 
 *
 ******************************************************************************/
 
@@ -34,6 +36,7 @@
 ******************************************************************************/
 
 #define CRE CDF_READ_ERROR
+#define CV2C CORRUPTED_V2_CDF 
 
 /******************************************************************************
 * Read32.
@@ -252,6 +255,8 @@ va_dcl
       if (!Read32(fp,&(fp->GDR->rfuC))) return CRE;
       if (!Read32(fp,&(fp->GDR->rfuD))) return CRE;
       if (!Read32(fp,&(fp->GDR->rfuE))) return CRE;
+      if (fp->GDR->rNumDims < 0 || fp->GDR->rNumDims > CDF_MAX_DIMS)
+        return CV2C;  
       if (!Read32s(fp,fp->GDR->rDimSizes,
                    (int)fp->GDR->rNumDims)) return CRE;
       pStatus = ReadADRList(fp);
@@ -287,6 +292,8 @@ va_dcl
              if (!Read32(fp,&(GDR->rfuC))) return CRE;
              if (!Read32(fp,&(GDR->rfuD))) return CRE;
              if (!Read32(fp,&(GDR->rfuE))) return CRE;
+             if (GDR->rNumDims < 0 || GDR->rNumDims > CDF_MAX_DIMS)
+               break; /* return CV2C;  */
              if (!Read32s(fp,GDR->rDimSizes,(int)GDR->rNumDims)) return CRE;
          }
          else
@@ -322,6 +329,8 @@ va_dcl
                              GDR_NULL),&pStatus)) return pStatus;
 	     tOffset = offset + GDR_rDIMSIZES_OFFSET;
 	     if (!SEEKv(fp,tOffset,vSEEK_SET)) return CRE;
+             if (rNumDims < 0 || rNumDims > CDF_MAX_DIMS)
+               return CV2C;
 	     if (!Read32s(fp,rDimSizes,(int)rNumDims)) return CRE;
          }
          else
@@ -560,7 +569,8 @@ va_dcl
              ADR->NzEntries = fp->ADRList[fp->CurADRIndex]->NzEntries;
              ADR->MAXzEntry = fp->ADRList[fp->CurADRIndex]->MAXzEntry;
              ADR->rfuE = fp->ADRList[fp->CurADRIndex]->rfuE;
-             strcpy(ADR->Name, fp->ADRList[fp->CurADRIndex]->Name);
+             strcpyX (ADR->Name, fp->ADRList[fp->CurADRIndex]->Name,
+                      CDF_ATTR_NAME_LEN);
          }
          break;
        }
@@ -575,7 +585,8 @@ va_dcl
          }
          else
          {
-             strcpy(aName, fp->ADRList[fp->CurADRIndex]->Name);
+             strcpyX (aName, fp->ADRList[fp->CurADRIndex]->Name,
+                      CDF_ATTR_NAME_LEN);
          }
 	 break;
        }
@@ -1018,6 +1029,8 @@ va_dcl
 	 NulPad (VDR->Name, CDF_VAR_NAME_LEN);
 	 if (zVar) {
 	   if (!Read32(fp,&(VDR->zNumDims))) return CRE;
+           if (VDR->zNumDims < 0 || VDR->zNumDims > CDF_MAX_DIMS)
+             return CV2C;
 	   if (!Read32s(fp,VDR->zDimSizes,(int)VDR->zNumDims)) return CRE;
 	 }
 	 if (zVar)
@@ -1032,6 +1045,8 @@ va_dcl
 			   GDR_NULL),&pStatus)) return pStatus;
 	   if (!SEEKv(fp,tOffset,vSEEK_SET)) return CRE;
 	 }
+         if (nDims < 0 || nDims > CDF_MAX_DIMS)
+           return CV2C;
 	 if (!Read32s(fp,VDR->DimVarys,(int)nDims)) return CRE;
 	 if (PADvalueBITset(VDR->Flags) && padValue != NULL) {
 	   size_t nBytes = (size_t) (CDFelemSize(VDR->DataType)*VDR->NumElems);
@@ -1067,6 +1082,8 @@ va_dcl
 	 tOffset = offset + zVDR_zDIMSIZES_OFFSET +
 		   BOO(CDF->wastedSpace,VDR_WASTED_SIZE,0);
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CRE;
+         if (zNumDims < 0 || zNumDims > CDF_MAX_DIMS)
+           return CV2C;
 	 if (!Read32s(fp,zDimSizes,(int)zNumDims)) return CRE;
 	 break;
        }
@@ -1091,6 +1108,8 @@ va_dcl
 	   tOffset = offset + rVDR_DIMVARYS_OFFSET +
 		     BOO(CDF->wastedSpace,VDR_WASTED_SIZE,0);
 	 }
+         if (nDims < 0 || nDims > CDF_MAX_DIMS)
+           return CV2C;
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CRE;
 	 if (!Read32s(fp,dimVarys,(int)nDims)) return CRE;
 	 break;
@@ -1201,6 +1220,8 @@ va_dcl
 	 if (!Read32(fp,&(VXR->VXRnext))) return CRE;
 	 if (!Read32(fp,&(VXR->Nentries))) return CRE;
 	 if (!Read32(fp,&(VXR->NusedEntries))) return CRE;
+         if (VXR->Nentries < 0 || VXR->Nentries > MAX_VXR_ENTRIES)
+           return CV2C;
 	 if (!Read32s(fp,VXR->First,(int)VXR->Nentries)) return CRE;
 	 if (!Read32s(fp,VXR->Last,(int)VXR->Nentries)) return CRE;
 	 if (!Read32s(fp,VXR->Offset,(int)VXR->Nentries)) return CRE;
@@ -1221,6 +1242,8 @@ va_dcl
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CRE;
+         if (nEntries < 0 || nEntries > MAX_VXR_ENTRIES)
+           return CV2C;
 	 if (!Read32s(fp,buffer,(int)nEntries)) return CRE;
 	 break;
        }
@@ -1648,7 +1671,7 @@ STATICforIDL CDFstatus ReadADRList (vFILE *fp)
 STATICforIDL CDFstatus ReadAEDRList (vFILE *fp, 
                                      struct AEDRstructExt ***AEDRList, 
                                      Int32 AEDRHead, 
-                                     Int32 NumEntries,
+                                     Int32 NumEntries, 
                                      Int32 MaxEntry)
 {
   CDFstatus pStatus = CDF_OK;
@@ -1699,10 +1722,13 @@ STATICforIDL CDFstatus ReadAEDRList (vFILE *fp,
       if (!Read32(fp,&(TempAEDR->AEDR.rfuE))) return CRE;
       TempAEDR->ValueSize = (CDFelemSize(TempAEDR->AEDR.DataType) *
                              TempAEDR->AEDR.NumElems);
+      if (TempAEDR->ValueSize < 1) return CV2C;
       TempAEDR->Value = cdf_AllocateMemory(TempAEDR->ValueSize, NULL);
       if (TempAEDR->Value == NULL) return BAD_MALLOC;
       if (!READv(TempAEDR->Value,TempAEDR->ValueSize,1,fp)) return CRE;
-      ++iEntries; 
+      ++iEntries;
+      if (TempAEDR->AEDR.Num < 0 || TempAEDR->AEDR.Num > MaxEntry)
+        return CV2C;
       (*AEDRList)[TempAEDR->AEDR.Num] = TempAEDR;
   };
   return pStatus;
