@@ -36,6 +36,10 @@
 *                               nCACHE_BUFFER_BYTEs.
 *   V4.10 20-Jun-07, D Berger   Added initialization of variables to support
 *                               READONLYon enhancements.
+*   V4.11 18-Jun-08, M Liu      Modified the V_read and V_write to make sure
+*                               the actual read/write bytes is within the 
+*                               buffer size.
+*   V4.12 12-Aug-10, M Liu      Fill 0s to each new cache in AllocateBuffer.
 *
 ******************************************************************************/
 
@@ -107,8 +111,9 @@ char *a_mode;
 #if defined(vms)
   char mrs[10+1];       /* Maximum record size. */
   char deq[10+1];       /* Default allocation quantity. */
-  sprintf (mrs, "mrs=%d", nCACHE_BUFFER_BYTEs);
-  sprintf (deq, "deq=%d", VMS_DEFAULT_nALLOCATION_BLOCKS);
+  snprintf (mrs, (size_t) sizeof(mrs), "mrs=%d", nCACHE_BUFFER_BYTEs);
+  snprintf (deq, (size_t) sizeof(deq), "deq=%d",
+	    VMS_DEFAULT_nALLOCATION_BLOCKS);
   return fopen(file_spec,a_mode,"rfm=fix",mrs,deq);
 #else
   return FOPEN(file_spec,a_mode);
@@ -237,6 +242,7 @@ vFILE *vFp;
 	  cache->prev = NULL;
 	}
 	(vFp->nBuffers)++;
+	memset(cache->ptr, 0, nCACHE_BUFFER_BYTEs);
 	return cache;
       }
       else {
@@ -293,6 +299,7 @@ vFILE *vFp;
     cache->prev = NULL;
   }
   (vFp->nPageOuts)++;
+  memset(cache->ptr, 0, nCACHE_BUFFER_BYTEs);
   return cache;
 }
 
@@ -493,7 +500,8 @@ vFILE *vFp;
     for (i = 1; i <= MAX_TMP; i++) {
        strcpyX (tmpPath, vFp->path, 0);
        AppendToDir (tmpPath, "");
-       sprintf (EofS(tmpPath), "TMP%05ld.%s", i, vFp->scratchExt);
+       snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                 "TMP%05ld.%s", i, vFp->scratchExt);
        if (!IsReg(tmpPath)) {
 	 FILE *fp = OpenFile (tmpPath, WRITE_PLUS_a_mode);
 	 if (fp == NULL) {
@@ -577,7 +585,8 @@ char *a_mode;           /* Access mode. */
 	fclose (fp);
 	return NULL;
       }
-      for (i = 0; i < nCACHE_BUFFER_BYTEs; i++) buffer[i] = 0;
+      /* for (i = 0; i < nCACHE_BUFFER_BYTEs; i++) buffer[i] = 0; */
+      memset (buffer, 0, (size_t) nCACHE_BUFFER_BYTEs);
       if (fread(buffer,EOFoffsetInBlock,1,fp) != 1) {
 	fclose (fp);
 	return NULL;
@@ -867,6 +876,7 @@ size_t item_size;       /* Size (in bytes) of each item to read. */
 size_t n_items;         /* Number of items to read. */
 vFILE *vFp;             /* Pointer to vFILE structure. */
 {
+  size_t nBytesX;       /* Total number of bytes in buffer. */
   size_t nBytes;        /* Total number of bytes to read. */
   long remainingItems;	/* Number of items remaining after the offset. */
   size_t nItems;	/* Number of items to actually be read. */
@@ -903,6 +913,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   else
     nItems = n_items;
   nBytes = nItems * item_size;
+  nBytesX = n_items * item_size;
   (vFp->nV_reads)++;
   /****************************************************************************
   * Read from first block...
@@ -911,6 +922,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   bufferOffset = (int) (vFp->offset % nCACHE_BUFFER_BYTEs);
   remainingBytes = nCACHE_BUFFER_BYTEs - bufferOffset;
   xBytes = MINIMUM (nBytes, (size_t) remainingBytes);
+  if (xBytes > nBytesX) return 0;
   if (bufferOffset > 0 || xBytes < nCACHE_BUFFER_BYTEs) {
     cache = FindCache (vFp, firstBlockN);
     if (cache == NULL) cache = PageIn (vFp, firstBlockN);
@@ -936,6 +948,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   for (blockN = atBlockN; blockN <= lastBlockN; blockN++) {
      xBytes = (size_t) (vFp->offset + nBytes - (nCACHE_BUFFER_BYTEs * blockN));
      xBytes = MINIMUM (xBytes, nCACHE_BUFFER_BYTEs);
+     if (xBytes > nBytesX) return 0;
      cache = FindCache (vFp, blockN);
      if (cache != NULL) {
        cBuffer = CACHEbufferREADfrom (cache);
@@ -986,6 +999,7 @@ size_t item_size;       /* Size (in bytes) of each item to write. */
 size_t n_items;         /* Number of items to write. */
 vFILE *vFp;             /* Pointer to vFILE structure. */
 {
+  size_t nBytesX;       /* Total number of bytes in buffer. */
   size_t nBytes;        /* Total number of bytes in write. */
   long firstBlockN;     /* First block involved in write. */
   long lastBlockN;      /* Last block involved in write. */
@@ -1008,6 +1022,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   if ((int) item_size <= 0 || (int) n_items <= 0) return 0;
   vFp->eof = FALSE;		/* Cleared before proceeding. */
   nBytes = item_size * n_items;
+  nBytesX = n_items * item_size;
   if (nBytes < 1) return 0;
   (vFp->nV_writes)++;
   /****************************************************************************
@@ -1019,6 +1034,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   bufferOffset = (int) (vFp->offset % nCACHE_BUFFER_BYTEs);
   nBytesInBlock = nCACHE_BUFFER_BYTEs - bufferOffset;
   xBytes = MINIMUM (nBytes, nBytesInBlock);
+  if (xBytes > nBytesX) return 0;
   if (vFp->scratch || bufferOffset > 0 || xBytes < nCACHE_BUFFER_BYTEs) {
     cache = FindCache (vFp, firstBlockN);
     if (cache == NULL) {
@@ -1068,6 +1084,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   for (blockN = atBlockN; blockN <= lastBlockN; blockN++) {
      xBytes = (size_t) (vFp->offset + nBytes - (nCACHE_BUFFER_BYTEs * blockN));
      xBytes = MINIMUM (xBytes, nCACHE_BUFFER_BYTEs);
+     if (xBytes > nBytesX) return 0;
      /*************************************************************************
      * Is this block in the cache?  If so, move the number of bytes to be
      * written at this block to its cache buffer.
