@@ -64,6 +64,7 @@ C-----------------------------------------------------------------------
      .		  cReserveOUT
         INTEGER*4 recBuffer1(2,3), recBuffer2(2,3), recBuffer1OUT(2,3), 
      .		  recBuffer2OUT(2,3)
+        INTEGER*4 recBuffer1x(3), recBuffer1xOUT(3)
 
 	CHARACTER CDFNAME*(CDF_PATHNAME_LEN),
      .		  VARNAME*(CDF_VAR_NAME_LEN256),
@@ -99,7 +100,8 @@ C-----------------------------------------------------------------------
      .	     cParms(1)/RLE_OF_ZEROs/, sRecordsType/PAD_SPARSERECORDS/,
      .	     sArraysType/NO_SPARSEARRAYS/, cReserve/150/,
      .	     recBuffer1/1,2,3,4,5,6/,
-     .       recBuffer2/7,8,9,10,11,12/
+     .       recBuffer2/7,8,9,10,11,12/,
+     .       recBuffer1x/13,14,15/
 
 C-----------------------------------------------------------------------
 C NUL-terminate character strings.
@@ -413,6 +415,20 @@ C-----------------------------------------------------------------------
 	  END DO
 	END DO
 
+        CALL CDF_put_zvar_recorddata (ID, varNums(1),
+     .                                11, recBuffer1x, STATUS)
+        IF (STATUS .LT. CDF_OK) CALL QUIT_CDF (STATUS, '17.5')
+
+        CALL CDF_get_zvar_recorddata (ID, varNums(1),
+     .                                11, recBuffer1xOUT, STATUS)
+        IF (STATUS .LT. CDF_OK) CALL QUIT_CDF (STATUS, '17.6')
+
+        DO J = 1, 3
+          IF (recBuffer1xOUT(j) .NE. recBuffer1x(j)) THEN
+            CALL QUIT_CDF (STATUS, '17.7')
+          ENDIF
+        END DO
+
 C-----------------------------------------------------------------------
 C Create attribute.
 C-----------------------------------------------------------------------
@@ -496,7 +512,7 @@ C-----------------------------------------------------------------------
 
 	IF (ENCODING_OUT .NE. ENCODING) CALL QUIT_CDF (STATUS, '22.4')
 	IF (MAJORITY_OUT .NE. MAJORITY) CALL QUIT_CDF (STATUS, '22.5')
-	IF (MAX_ZREC_OUT .NE. 10) CALL QUIT_CDF (STATUS, '22.6')
+	IF (MAX_ZREC_OUT .NE. 11) CALL QUIT_CDF (STATUS, '22.6')
 	IF (NUM_ZVARS_OUT .NE. 3) CALL QUIT_CDF (STATUS, '22.7')
 	IF (NUM_ATTRS_OUT .NE. 1) CALL QUIT_CDF (STATUS, '22.8')
 	IF (cTypeOUT .NE. cType) CALL QUIT_CDF (STATUS, '22.9')
@@ -698,13 +714,17 @@ C QUIT_CDF.  Abort test early due to CDF error.
 C-----------------------------------------------------------------------
 
 	SUBROUTINE QUIT_CDF (STATUS, WHERE)
+	INCLUDE 'CDF.INC'
 	INTEGER*4 STATUS
 	CHARACTER WHERE*(*)
+	CHARACTER TEXT*(CDF_ERRTEXT_LEN)
+	INTEGER*4 STATUS2
 	WRITE (6,401) WHERE
  401	FORMAT (' ', 'Aborting at ', A, '...')
 	IF (STATUS .LT. 0) THEN
-	  WRITE (6,501) STATUS
- 501	  FORMAT (' ', 'CDF status code: ', I5)
+	  CALL CDF_ERROR (STATUS, TEXT, STATUS2)
+	  WRITE (6,501) TEXT
+ 501	  FORMAT (' ', 'ERROR: ', A)
 	ENDIF
 	WRITE (6,404)
  404	FORMAT (' ','...test aborted')
