@@ -229,10 +229,13 @@ static char zVarsRecBufferOut[zDIM_0_SIZEa][zNUM_ELEMSa];
 
 printf ("Testing Internal/C interface...\n");
 
+/* CDFsetFileBackward(BACKWARDFILEon); */
+/* CDFsetValidate(VALIDATEFILEon); */
+
 /******************************************************************************
 * Create CDF.
 ******************************************************************************/
-/* CDFsetFileBackward(BACKWARDFILEon); */
+
 status = CDFlib (CREATE_, CDF_, "TEST", numDims, dimSizes, &id,
 		 PUT_, CDF_ENCODING_, encoding,
 		       CDF_MAJORITY_, majority,
@@ -941,7 +944,7 @@ status = CDFlib (SELECT_, ATTR_, 0L,
 if (status < CDF_OK) QuitCDF ("27.0", status);
 
 /******************************************************************************
-* Delete entries/attribute/variables.
+* Delete entries/attribute/variables and variable record.
 ******************************************************************************/
 
 status = CDFlib (SELECT_, ATTR_, 0L,
@@ -959,9 +962,11 @@ if (status < CDF_OK) QuitCDF ("28.0", status);
 status = CDFlib (SELECT_, ATTR_, 0L,
 		 DELETE_, ATTR_, 
 		 SELECT_, rVAR_, 0L,
-		 DELETE_, rVAR_, 
+		 DELETE_, rVAR_RECORDS_, 0L, 0L,
+                 SELECT_, rVAR_, 1L,
+		 DELETE_, rVAR_RECORDS_, 2L, 2L,
 		 SELECT_, zVAR_, 0L,
-		 DELETE_, zVAR_, 
+		 DELETE_, zVAR_RECORDS_, 1L, 1L,
 		 NULL_);
 if (status < CDF_OK) QuitCDF ("28.1", status);
 
@@ -973,6 +978,18 @@ status = CDFlib (CLOSE_, CDF_,
 		 NULL_);
 if (status < CDF_OK) QuitCDF ("29.0", status);
 
+/******************************************************************************
+* Reopen and close CDF to validate the file.
+******************************************************************************/
+
+status = CDFlib (OPEN_, CDF_, "TEST", &id,
+                 NULL_);
+if (status < CDF_OK) QuitCDF ("36.0", status);
+
+status = CDFlib (CLOSE_, CDF_,
+                 NULL_);
+if (status < CDF_OK) QuitCDF ("37.0", status);
+
 /******************************************************************************
 * Successful completion.
 ******************************************************************************/
