@@ -7,6 +7,7 @@
 *  Modification history:
 *
 *   V1.0  09-Jul-06, M Liu      Original version.
+*   V3.3  10-Jan-09, M Liu      Validate a file before it is used.
 *
 ******************************************************************************/
 
@@ -55,7 +56,7 @@ char *argV[];
 CDFid id;
 CDFstatus status;
 long numDims, dimSizes[CDF_MAX_DIMS], dimIntervals[CDF_MAX_DIMS], 
-     dimIndices[CDF_MAX_DIMS];
+     dimIndices[CDF_MAX_DIMS], dimCounts[CDF_MAX_DIMS];
 long version, release, increment, format, majority, encoding, numrVars,
      numzVars, numAttrs, numgAttrs, numvAttrs, numgEntries, numrEntries, 
      numzEntries, checksum;
@@ -88,10 +89,10 @@ FILE *fp;
 QOP *qop;
 static char *validQuals[] = {
   "dump", "output", "format", "noformat", "vars", "about", "noheader", 
-  NULL 
+  "recordrange", NULL 
 };
 static int optRequired[] = {
-  TRUE, TRUE, FALSE, FALSE, TRUE, FALSE, FALSE, 
+  TRUE, TRUE, FALSE, FALSE, TRUE, FALSE, FALSE, TRUE, 
   0
 };
 int numValidDumps = 3;
@@ -123,7 +124,7 @@ switch (argC) {
       strcpyX (dumpOption, qop->qualOpt[DUMPqual], 20);
       Z = FALSE;
       for (i = 0; i < numValidDumps; i++) {
-        if (strcmpIgCase(validDumps[i], dumpOption) != 0) {
+        if (strcmpIgCase(validDumps[i], dumpOption) == 1) {
           dump = i;
           Z = TRUE;
           break;
@@ -158,9 +159,14 @@ switch (argC) {
     /************************************************************************
     * Check for `output' qualifier.
     ************************************************************************/
-    if (qop->qualEntered[OUTPUTqual])
-      strcpyX (outputFile, qop->qualOpt[OUTPUTqual], CDF_PATHNAME_LEN);
-    else
+    if (qop->qualEntered[OUTPUTqual]) {
+      if (!ValidateQual(qop->qualOpt[OUTPUTqual], validQuals))
+        strcpyX (outputFile, qop->qualOpt[OUTPUTqual], CDF_PATHNAME_LEN);
+      else {
+        DisplayError ("Invalid output option.");
+        qopError = TRUE;
+      }
+    } else
       MakeNUL(outputFile);
     /************************************************************************
     * Check for `vars' qualifier.
@@ -178,13 +184,26 @@ switch (argC) {
       }   
     } else
       MakeNUL(varsOption);
+    /*********************************************************************
+    * Check for `recordrange' qualifier.
+    *********************************************************************/
+    if (qop->qualEntered[RECORDRANGEqual]) {
+      if (!ParseRecordRange(qop->qualOpt[RECORDRANGEqual],
+                            &startingRec,&endingRec)) {
+        DisplayError ("Illegal record range.");
+        qopError = TRUE;
+      }
+    }
+    else {
+      startingRec = useDEFAULTrecordNUM;
+      endingRec = useDEFAULTrecordNUM;
+    }
     /**************************************************************************
     * Get CDF pathname.
     **************************************************************************/
     if (qop->Nparms < 1) {
       DisplayError ("Missing parameter.");
       qopError = TRUE;
-      return TRUE;
     }
     else { 
       strcpyX (CDFpath, qop->parms[CDFPATHparm], DU_MAX_PATH_LEN);
@@ -195,17 +214,28 @@ switch (argC) {
       if (strncmpIgCase(outputFile, "source", 6) != 0) {
         int loc;
         char mytmp[CDF_PATHNAME_LEN+1];
-        strcpy(mytmp, CDFpath);
+        strcpyX (mytmp, CDFpath, CDF_PATHNAME_LEN);
         loc = StrlaststrIgCase(mytmp, ".cdf");
         if (loc == -1) strcatX (EofS(mytmp), ".txt", 0);
         else strcpyX (mytmp+loc, ".txt", 0);
         strcpyX (outputFile, mytmp, 0);
       } else {
         ParsePath (outputFile, oDir, oName);
-        if (strchr(oName,'.') == NULL) strcatX (outputFile, ".txt", DU_MAX_PATH_LEN);
+        if (strchr(oName,'.') == NULL) 
+          strcatX (outputFile, ".txt", DU_MAX_PATH_LEN);
       }
     }
-  }
+    if (!strcmp(outputFile, CDFpath)) {
+      DisplayError ("Output file name is identical to source CDF file.");
+      qopError = TRUE;
+    }
+    /**************************************************************************
+    * Free QOP memory and check for an error.
+    **************************************************************************/
+    cdf_FreeMemory (qop, FatalError);
+    if (qopError) return FALSE;
+    break;
+}
 
   /****************************************************************************
   * Display information.
@@ -222,12 +252,7 @@ switch (argC) {
     WriteOut (stdout, "\"...\n");
   }
 
-  if (!strcmp(outputFile, CDFpath)) {
-      DisplayError ("Output file name is identical to source CDF file.");
-      qopError = TRUE;
-      return TRUE;
-  }
-
+  CDFsetValidate (VALIDATEFILEon);
   /****************************************************************************
   * Open/inquire CDF.
   ****************************************************************************/
@@ -237,6 +262,7 @@ switch (argC) {
   if (status < CDF_OK) QuitCDF ("1.0", status, CDFpath);
 
   status = CDFlib (SELECT_, CDF_, id,
+                            CDF_READONLY_MODE_, READONLYon,
                    GET_, CDF_VERSION_, &version,
                          CDF_RELEASE_, &release,
                          CDF_INCREMENT_, &increment,
@@ -274,7 +300,7 @@ switch (argC) {
                                                                    numvAttrs);
     fprintf (fp, "Compression:  %s\n",CompressionToken(compression, 
                                                        compressParms));
-    if (!PriorTo ("3.2.0", version, release, increment)) 
+    if (!PriorTo ("3.2.0", (Int32) version, (Int32) release, (Int32) increment)) 
       fprintf (fp, "Checksum:     %s\n", ChecksumToken(checksum));
   }
 
@@ -331,9 +357,20 @@ switch (argC) {
                            j,DataTypeToken(dataTypeX),numElemsX);
               if (STRINGdataType(dataTypeX)) {
                 ((char *)value)[numElemsX] = '\0';
-                fprintf (fp, "%s\n", value);
+                fprintf (fp, "\"%s\"\n", (char *)value);
               } else {
-                EncodeValue(dataTypeX, value, tText, 0);
+                int ix, style;
+                char tmp[256];
+                if (TT2000dataType(dataTypeX)) style = TT2000_3_STYLE;
+                else style = EPOCH0_STYLE;
+                for (ix = 0; ix < numElemsX; ++ix) {
+                  EncodeValue (dataTypeX, (char *)value+ix*CDFelemSize(dataTypeX),
+                               tmp, style, (size_t) sizeof(tmp));
+                  if (ix == 0) strcpyX (tText, tmp, 0);
+                  else strcatX (tText, tmp, 255);
+                  if (numElemsX > 1 && ix < (numElemsX-1))
+                    strcatX (tText, " ", 255);
+                }
                 fprintf (fp, "%s\n", tText);
               }
               k++;
@@ -431,12 +468,18 @@ switch (argC) {
       else {
         for (j = 0; j < numDims; j++) {
           if (j == 0) {
-            sprintf (shortStr1, "%ld", dimSizes[j]);
-            sprintf (shortStr2, "%s", BOO(dimVarys[j],"T","F"));
+            snprintf (shortStr1, (size_t) sizeof(shortStr1),
+		      "%ld", dimSizes[j]);
+            snprintf (shortStr2, (size_t) sizeof(shortStr2), 
+		      "%s", BOO(dimVarys[j],"T","F"));
           } else {
             strcat(shortStr1, ",");
-            sprintf (EofS(shortStr1), "%ld", dimSizes[j]);
-            sprintf (EofS(shortStr2), "%s", BOO(dimVarys[j],"T","F"));
+            snprintf (EofS(shortStr1),
+		      (size_t) sizeof(shortStr1)-strlen(shortStr1), "%ld", 
+		      dimSizes[j]);
+            snprintf (EofS(shortStr2),
+		      (size_t) sizeof(shortStr2)-strlen(shortStr2), "%s", 
+		      BOO(dimVarys[j],"T","F"));
           }
         }
         fprintf (fp, "CDF_%s/%ld\t%ld:[%s]\t%s/%s", 
@@ -486,12 +529,18 @@ switch (argC) {
       else {
         for (j = 0; j < numDims; j++) {
           if (j == 0) {
-            sprintf (shortStr1, "%ld", dimSizes[j]);
-            sprintf (shortStr2, "%s", BOO(dimVarys[j],"T","F"));
+            snprintf (shortStr1, (size_t) sizeof(shortStr1), "%ld",
+		      dimSizes[j]);
+            snprintf (shortStr2, (size_t) sizeof(shortStr2), "%s",
+		      BOO(dimVarys[j],"T","F"));
           } else {
             strcat(shortStr1, ",");
-            sprintf (EofS(shortStr1), "%ld", dimSizes[j]);
-            sprintf (EofS(shortStr2), "%s", BOO(dimVarys[j],"T","F"));
+            snprintf (EofS(shortStr1),
+		      (size_t) sizeof(shortStr1)-strlen(shortStr1), "%ld", 
+		      dimSizes[j]);
+            snprintf (EofS(shortStr2),
+		      (size_t) sizeof(shortStr2)-strlen(shortStr2), "%s", 
+		      BOO(dimVarys[j],"T","F"));
           }
         }
         fprintf (fp, "CDF_%s/%ld\t%ld:[%s]\t%s/%s", 
@@ -574,7 +623,10 @@ switch (argC) {
           ((char *)value)[numElems] = '\0';
           strcpyX (tText, value, numElems);
         } else {
-          EncodeValue(dataType, value, tText, 0);
+          int style;
+          if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
+          else style = EPOCH0_STYLE;
+          EncodeValue (dataType, value, tText, style, (size_t) sizeof(tText));
         }
       } else
         pad = FALSE;
@@ -616,12 +668,18 @@ switch (argC) {
       else {
         for (j = 0; j < numDims; j++) {
           if (j == 0) {
-            sprintf (shortStr1, "%ld", dimSizes[j]);
-            sprintf (shortStr2, "%s", BOO(dimVarys[j],"T","F"));
+            snprintf (shortStr1, (size_t) sizeof(shortStr1), "%ld",
+		      dimSizes[j]);
+            snprintf (shortStr2, (size_t) sizeof(shortStr2), "%s",
+		      BOO(dimVarys[j],"T","F"));
           } else {
             strcat(shortStr1, ",");
-            sprintf (EofS(shortStr1), "%ld", dimSizes[j]);
-            sprintf (EofS(shortStr2), "%s", BOO(dimVarys[j],"T","F"));
+            snprintf (EofS(shortStr1),
+		      (size_t) sizeof(shortStr1)-strlen(shortStr1), "%ld", 
+		      dimSizes[j]);
+            snprintf (EofS(shortStr2),
+		      (size_t) sizeof(shortStr2)-strlen(shortStr2), "%s", 
+		      BOO(dimVarys[j],"T","F"));
           }
         }
         fprintf (fp, "%ld:[%s]\t(%s/%s)", numDims, shortStr1,
@@ -632,16 +690,20 @@ switch (argC) {
       if (useFormat && !NULstring(prtFormat))
         fprintf (fp, "Format:              %s\n", prtFormat);
       if (dump != DATADUMP)
-        fprintf (fp, "Compression:         %s\n", CompressionToken(compression, 
-                                                                compressParms));
+        if (compression != NO_COMPRESSION) {
+          fprintf (fp, "Compression:         %s\n", 
+                       CompressionToken(compression, compressParms));
+        }
       if (pad) fprintf (fp, "Pad value:           %s\n", tText);
       fprintf (fp, "Written Records:     %ld/%ld(max)\n", numRecs, (maxRec+1));
       if (dump != DATADUMP) {
         fprintf (fp, "Allocated Records:   %ld/%ld(max)\n", numAllocRecs, 
                                                         (maxAllocRec+1));
-        fprintf (fp, "Blocking Factor:     %ld\n", bf);
+        fprintf (fp, "Blocking Factor:     %ld (records)\n", bf);
       }
-      fprintf (fp, "Sparseness:          %s\n", SparsenessToken(sp, 0L, tmpLong));
+      if (sp != NO_SPARSERECORDS)
+        fprintf (fp, "Sparseness:          %s\n", 
+                     SparsenessToken(sp, 0L, tmpLong));
     }
 
     if (dump != DATADUMP) {
@@ -681,10 +743,26 @@ switch (argC) {
                        DataTypeToken(dataTypeX),numElemsX);
           if (STRINGdataType(dataTypeX)) {
             ((char *)value)[numElemsX] = '\0';
-            fprintf (fp, "\"%s\"\n", value);
+            fprintf (fp, "\"%s\"\n", (char *)value);
           } else {
-            EncodeValue(dataTypeX, value, tText, 0);
-            fprintf (fp, "%s\n", tText);
+            int style;
+            if (TT2000dataType(dataTypeX)) style = TT2000_3_STYLE;
+            else style = EPOCH0_STYLE;
+            if (numElemsX > 1) {
+              int ip;
+              int ix = CDFelemSize(dataTypeX);
+              for (ip = 0; ip < numElemsX; ++ip) {
+                EncodeValue (dataTypeX, ((char *) value)+ix*ip, tText, style,
+			     (size_t) sizeof(tText));
+                fprintf (fp, "%s", tText);
+                if (ip != (numElemsX -1)) fprintf (fp, ", ");
+              }
+              fprintf (fp, "\n");
+            } else {
+              EncodeValue (dataTypeX, value, tText, style,
+                           (size_t) sizeof(tText));
+              fprintf (fp, "%s\n", tText);
+            }
           }
           cdf_FreeMemory (value, FatalError);
         }
@@ -713,23 +791,36 @@ switch (argC) {
       nValueBytes[0] = (size_t) numElems * CDFelemSize(dataType);
 
       if (sp == NO_SPARSERECORDS) {
-  
+        long toRecs, ist, ien;
+        ist = startingRec - 1;
+        ien = endingRec - 1;
         handles[0] = &buffer;
-        AllocateBuffers (maxRec + 1, numDims, dimSizes, &groups, 0, 1, handles,
+        if (ist != -1 && ist > maxRec) {
+          fprintf (fp, "\n");
+          continue;
+        }
+        if (ien != -1 && ien > maxRec) ien = maxRec;
+        if (ist == -1) 
+          toRecs = maxRec + 1;
+        else
+          toRecs = (ien == -1 ? maxRec : ien) - ist + 1;
+        AllocateBuffers (toRecs, numDims, dimSizes, &groups, 0, 1, handles,
                          nValueBytes, ROWmajor(majority), 5, FatalError);
-        InitHyperParms (&hyperX, &groups, numDims, &nHypers, &nValues);
+        InitHyperParmsFromAnywhere (&hyperX, &groups, numDims, &nHypers, 
+                                    &nValues, (ist == -1 ? 0 : ist));
 
         if (STRINGdataType(dataType)) imore = 1;
         else imore = 0;
         nBytes = (size_t) (CDFelemSize(dataType) * (numElems + imore));
         working = cdf_AllocateMemory ((size_t) nBytes, FatalError);
-
         for (hyperN = 0; hyperN < nHypers; hyperN++) {
+          for (j = 0; j < numDims; ++j)
+            if (!dimVarys[j]) hyperX.dimCounts[j] = 1;
           status = HYPER_READ (id, Z, hyperX, buffer);
           if (status < CDF_OK) QuitCDF("16.0", status, varName);
           offset = 0;
           for (k = 0, value = buffer; k < nValues;
-               k += iskip, offset += (size_t) (iskip * nValueBytes[0])) {
+               k += iskip, offset += (size_t) nValueBytes[0]) {
             i = hyperX.recNumber + k / nLogValuesPerRec; 
             j = k - k / nLogValuesPerRec * nLogValuesPerRec;
             if (j == 0) {
@@ -741,11 +832,17 @@ switch (argC) {
 
             if (STRINGdataType(dataType)) {
               ((char *)working)[numElems] = '\0';
-              fprintf (fp, "\"%s\"", working);
+              fprintf (fp, "\"%s\"", (char *)working);
             } else {
+              int style;
+              if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
+              else style = EPOCH0_STYLE;
               if (prtFormat != NULL) 
-              EncodeValueFormat(dataType, working, tText, prtFormat, 0, 0, 0);
-              else EncodeValue(dataType, working, tText, 0);
+                EncodeValueFormat (dataType, working, tText, prtFormat, 0, 0,
+                                   style, (size_t) sizeof(tText));
+              else
+		EncodeValue (dataType, working, tText, style,
+			     (size_t) sizeof(tText));
               fprintf (fp, "%s", tText);
             }
             if (nValuesPerRec == 1) {
@@ -770,6 +867,8 @@ switch (argC) {
         for (k = 0; k < numDims; k++) {
           dimIndices[k] = 0L;
           dimIntervals[k] = 1L;
+          if (dimVarys[k]) dimCounts[k] = dimSizes[k];
+          else dimCounts[k] = 1;
         }
         status = CDFlib(SELECT_, BOO(Z,zVAR_RECCOUNT_,rVARs_RECCOUNT_), 1L,
                                  BOO(Z,zVAR_RECINTERVAL_,rVARs_RECINTERVAL_), 
@@ -779,7 +878,7 @@ switch (argC) {
                                  BOO(Z,zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_), 
                                  dimIntervals,
                                  BOO(Z,zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_), 
-                                 dimSizes,
+                                 dimCounts,
                         NULL_);
         if (status < CDF_OK) QuitCDF("17.0", status, varName);
 
@@ -792,7 +891,7 @@ switch (argC) {
           if (status < CDF_OK) QuitCDF("18.0", status, varName);
           offset = 0;
           for (ir = 0, value = buffer; ir < nLogValuesPerRec;
-             ir += iskip, offset += (size_t) (iskip * nValueBytes[0])) {
+               ir += iskip, offset += (size_t) nValueBytes[0]) {
             if (ir == 0) {
               icount = 0;
               fprintf (fp, "  Record # %d: ",(k+1));
@@ -802,11 +901,17 @@ switch (argC) {
       
             if (STRINGdataType(dataType)) {
                 ((char *)working)[numElems] = '\0';
-              fprintf (fp, "\"%s\"", working);
+              fprintf (fp, "\"%s\"", (char *)working);
             } else {
+              int style;
+              if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
+              else style = EPOCH0_STYLE;
               if (prtFormat != NULL)
-                EncodeValueFormat(dataType, working, tText, prtFormat, 0, 0, 0);
-              else EncodeValue(dataType, working, tText, 0);
+                EncodeValueFormat (dataType, working, tText, prtFormat, 0, 0,
+                                   style, (size_t) sizeof(tText));
+              else
+		EncodeValue (dataType, working, tText, style,
+			     (size_t) sizeof(tText));
               fprintf (fp, "%s", tText);
             }
             if (nValuesPerRec == 1) {
@@ -831,6 +936,11 @@ switch (argC) {
 
   if (fp != stdout) fclose(fp);
 
+  if (numVars > 0) {
+    for (k = 0; k < numVars; ++k) 
+       cdf_FreeMemory (vars[k], FatalError);
+  }
+
   exit(1);
 
 }
