@@ -12,6 +12,7 @@
 *   V1.2  05-Aug-08, M Liu      Modified to handle variable "Epoch" 
 *                               differently from "Epoch_RANGE". "Epoch" is 
 *                               handled just like other variables.
+*   V3.3  10-Jan-09, M Liu      Validate a file before it is used.
 *
 ******************************************************************************/
 
@@ -67,9 +68,9 @@ static long maxRec, varMaxRec, sp, bf, maxAllocRec, fromRec, toRec, fromRec2, to
        nRecords, hyperStep;
 static long nValuesPerRec, maxGentry = 0;
 static char **CDFpaths, **varNames;
-static char outputCDF[CDF_PATHNAME_LEN+1], fileList[CDF_PATHNAME_LEN+1],
-       line[CDF_PATHNAME_LEN+1], sourceCDF[CDF_PATHNAME_LEN+1],
-       masterFile[CDF_PATHNAME_LEN+1], mytmp[CDF_PATHNAME_LEN+1];
+static char outputCDF[DU_MAX_PATH_LEN+1], fileList[DU_MAX_PATH_LEN+1],
+       line[DU_MAX_PATH_LEN+1], sourceCDF[DU_MAX_PATH_LEN+1],
+       masterFile[DU_MAX_PATH_LEN+1], mytmp[DU_MAX_PATH_LEN+1];
 static char varName[CDF_VAR_NAME_LEN256+1], attrName[CDF_ATTR_NAME_LEN256+1];
 static char newVarName[CDF_VAR_NAME_LEN256+1], newAttrName[CDF_ATTR_NAME_LEN256+1],
        newVarName2[CDF_VAR_NAME_LEN256+1];
@@ -137,6 +138,8 @@ char *argV[];
 time (&bintim);
 strftime (generationDate, 11, "%Y-%m-%d",gmtime(&bintim));
 
+CDFsetValidate (VALIDATEFILEon);
+
 /******************************************************************************
 * Parse qualifiers/options/parameters (QOP).
 ******************************************************************************/
@@ -189,16 +192,21 @@ switch (argC) {
     * Check for `master' qualifier.
     ************************************************************************/
     if (qop->qualEntered[MASTERqual]) {
-      master = TRUE;
-      dataOnly = TRUE;
-      strcpyX (masterFile, qop->qualOpt[MASTERqual], CDF_PATHNAME_LEN);
+      if (!ValidateQual(qop->qualOpt[MASTERqual], validQuals)) {
+        master = TRUE;
+        dataOnly = TRUE;
+        strcpyX (masterFile, qop->qualOpt[MASTERqual], DU_MAX_PATH_LEN);
+      } else {
+        DisplayError ("Invalid \"master\" qualifier.");
+        qopError = TRUE;
+      }
     }
     /************************************************************************
     * Check for `file' qualifier.
     ************************************************************************/
     if (qop->qualEntered[FILEqual]) {
-      strcpyX (fileList, qop->qualOpt[FILEqual], CDF_PATHNAME_LEN);
-#if defined(__CYGWIN__) || defined(__MINGW32__)
+      strcpyX (fileList, qop->qualOpt[FILEqual], DU_MAX_PATH_LEN);
+#if defined(__CYGWIN__)
       srcFp = fopen (fileList, "rt");
 #else
       srcFp = fopen (fileList, "r");
@@ -206,7 +214,6 @@ switch (argC) {
       if (srcFp == NULL) {
         printf ("*** Error! Unable to open the source file: %s.\n", fileList);
         qopError = TRUE;
-        return TRUE;
       }
       numCDFs = 0;
       while ( fgets(line,DU_MAX_PATH_LEN,srcFp) != NULL) {
@@ -219,7 +226,6 @@ switch (argC) {
         DisplayError ("Less than 3 files specified in the input file.");
         qopError = TRUE;
         fclose (srcFp);
-        return TRUE;
       }
 */
       rewind (srcFp);
@@ -250,19 +256,24 @@ switch (argC) {
           }
           if (prefixes != NULL) cdf_FreeMemory (prefixes, FatalError);
           if (CDFpaths != NULL) cdf_FreeMemory (CDFpaths, FatalError);
-          return TRUE;
         }
       }
       strcpyX (outputCDF, CDFpaths[numCDFs-1], strlen(line) - 1);
       fclose (srcFp);
       if (numOpt == 1) {
         for (i = 0; i < numCDFs; i++) 
-             if (prefixes[i] != NULL) cdf_FreeMemory (prefixes[i],
-                                                       FatalError);
+           if (prefixes[i] != NULL) cdf_FreeMemory (prefixes[i],
+                                                    FatalError);
         if (prefixes != NULL) cdf_FreeMemory (prefixes, FatalError);
       }
     } else
       MakeNUL (fileList);
+/*
+else {
+       DisplayError ("Invalid \"file\" qualifier.");
+       qopError = TRUE;
+    }
+*/
     /**************************************************************************
     * Get CDF pathnames if `file' qualifier is not entered.
     **************************************************************************/
@@ -271,7 +282,6 @@ switch (argC) {
       if (qop->Nparms < 3) {
         DisplayError ("Missing parameter(s)... Need at least 3 files (2 inputs 1 output).");
         qopError = TRUE;
-        return TRUE;
       }
       else { 
 */
@@ -312,7 +322,6 @@ switch (argC) {
             if (!master && (numTokens != (numCDFs-1))) {
               DisplayError ("Numbers of prefixes and source CDFs do not match.");
               qopError = TRUE;
-              return TRUE;
             } else {
               prefixes = (char **) cdf_AllocateMemory (sizeof(char *) * numTokens,
                                                        FatalError);
@@ -333,17 +342,26 @@ switch (argC) {
     ************************************************************************/
     cdf_FreeMemory (qop, FatalError);
     if (qopError) {
+      for (j = 0; j <= i; j++) {
+         if (prefixes[j] != NULL) cdf_FreeMemory (prefixes[j],
+                                                  FatalError);
+         if (CDFpaths[j] != NULL) cdf_FreeMemory (CDFpaths[j],
+                                                  FatalError);
+      }
+      if (prefixes != NULL) cdf_FreeMemory (prefixes, FatalError);
+      if (CDFpaths != NULL) cdf_FreeMemory (CDFpaths, FatalError);
       return FALSE;
     }
     break;
   }
 
-  /*****************************************/
-  /* Check the master file for information.
-  /*****************************************/
+  /*****************************************
+  * Check the master file for information.
+  *****************************************/
   if (master) {
     status = CDFlib (OPEN_, CDF_, masterFile, &idm,
                      SELECT_, CDF_zMODE_, zMODEon2,
+                              CDF_READONLY_MODE_, READONLYon,
                      GET_, CDF_MAJORITY_, &major,
                            CDF_ENCODING_, &encoding,
                            CDF_FORMAT_, &format,
@@ -391,7 +409,7 @@ switch (argC) {
   printf ("Merging cdf files: \"");
   for (icount = 0; icount < numCDFs; icount++) {
     if (icount != 0) printf ("        \"");
-    printf (CDFpaths[icount]);
+    printf ("%s", CDFpaths[icount]);
     if (!EndsWithIgCase (CDFpaths[icount], ".cdf"))
       if (mLog) 
         printf (".cdf\" (%d) \n", (icount+1));
@@ -404,20 +422,21 @@ switch (argC) {
         printf ("\" \n");
   }
   printf ("     into \"");
-  printf (outputCDF);
+  printf ("%s", outputCDF);
   if (!EndsWithIgCase (outputCDF, ".cdf"))
     printf (".cdf");
   printf ("\"\n");
 
-  /***********************************/
-  /* Loop through the source CDFs.   */
-  /***********************************/
+  /***********************************
+  * Loop through the source CDFs.   
+  ***********************************/
 
   for (icount = 0; icount < numCDFs; icount++) {
 
     if (!master || !masterCDF) 
       status = CDFlib (OPEN_, CDF_, CDFpaths[icount], &id,
                        SELECT_, CDF_zMODE_, zMODEon2,
+                                CDF_READONLY_MODE_, READONLYon,
                        GET_, CDF_MAJORITY_, &major,
                              CDF_ENCODING_, &encoding,
                              CDF_FORMAT_, &format,
@@ -430,6 +449,7 @@ switch (argC) {
     else 
       status = CDFlib (OPEN_, CDF_, CDFpaths[icount], &id,
                        SELECT_, CDF_zMODE_, zMODEon2,
+                                CDF_READONLY_MODE_, READONLYon,
                        GET_, CDF_NUMzVARS_, &numzVars,
                              CDF_NUMATTRS_, &numAttrs,
                              CDF_NUMvATTRS_, &numvAttrs,
@@ -437,9 +457,9 @@ switch (argC) {
     
     if (status < CDF_OK) QuitCDF ("1.0", status, CDFpaths[icount]);
 
-    /***************************************/
-    /* Create the output CDF.              */
-    /***************************************/
+    /***************************************
+    * Create the output CDF.              
+    ***************************************/
 
     if (icount == 0) {  
       majority = major;
@@ -454,9 +474,9 @@ switch (argC) {
 
       if (mLog) printf ("  Merged file created...\n");
 
-      /**************************************/
-      /* Add the "FileName" attribute.      */
-      /**************************************/
+      /**************************************
+      * Add the "FileName" attribute.      
+      **************************************/
 
       status = CDFlib (SELECT_, CDF_, ido,
                        CREATE_, ATTR_, "FileSource", GLOBAL_SCOPE, &attrNum,
@@ -465,9 +485,9 @@ switch (argC) {
 
     }
 
-    /********************************************/
-    /* Load the Global Attributes.              */
-    /********************************************/
+    /********************************************
+    * Load the Global Attributes.              
+    ********************************************/
 
     if (master) {
       if (icount == 0) {
@@ -493,14 +513,16 @@ switch (argC) {
       }
     }
 
-    /*******************************************************/
-    /* Add a file entry to "FileSource" attribute.
-    /*******************************************************/
+    /*******************************************************
+    * Add a file entry to "FileSource" attribute.         
+    *******************************************************/
 
     if (numTokens > 0) 
-      sprintf (entryValue, "%s: ", prefixes[icount]);
+      snprintf (entryValue, (size_t) sizeof(entryValue),
+	        "%s: ", prefixes[icount]);
     else
-      sprintf (entryValue, "File%d: ", (icount+1));
+      snprintf (entryValue, (size_t) sizeof(entryValue),
+	        "File%d: ", (icount+1));
     strcatX (entryValue, CDFpaths[icount], 0);
     if (!EndsWithIgCase (entryValue, ".cdf"))
       strcatX (entryValue, ".cdf", 0);
@@ -513,9 +535,9 @@ switch (argC) {
                      NULL_);
     if (status < CDF_OK) QuitCDF ("9.0", status, "FileSource");
 
-    /*******************************************************/
-    /* Load the variables (all are zVariables).           */ 
-    /*******************************************************/
+    /******************************************************
+    * Load the variables (all are zVariables).           
+    ******************************************************/
 
     if (numzVars > 0) {
 
@@ -545,15 +567,15 @@ switch (argC) {
         strcpyX (varNames[i], varName, strlen(varName));
 
 /*
-        if ((strcmpIgCase (varName, "epoch") != 0) ||
-            (strcmpIgCase (varName, "range_epoch") != 0)) {
+        if ((strcmpIgCase (varName, "epoch") == 1) ||
+            (strcmpIgCase (varName, "range_epoch") == 1)) {
 */
-        if (strcmpIgCase (varName, "range_epoch") != 0) {
+        if (strcmpIgCase (varName, "range_epoch") == 1) {
           if (first) {
-            /*******************************************************/
-            /* Create variable "Epoch" or "RANGE_EPOCH". -- old
-            /* Create variable "RANGE_EPOCH".
-            /*******************************************************/
+            /*******************************************************
+            * Create variable "Epoch" or "RANGE_EPOCH". -- old    
+            * Create variable "RANGE_EPOCH".                      
+            *******************************************************/
             dimSizes[0] = 0L;
             dimVarys[0] = 0L;
             strcpyX (epochName, varName, 0);
@@ -609,8 +631,8 @@ switch (argC) {
         if (status < CDF_OK) {
           if (status == VAR_EXISTS) {
             if (dataOnly || (cdaweb && 
-                             ((strcmpIgCase (newVarName, "EPOCH") != 0) ||
-                              (strcmpIgCase (newVarName, "RANGE_EPOCH") != 0))))
+                             ((strcmpIgCase (newVarName, "EPOCH") == 1) ||
+                              (strcmpIgCase (newVarName, "RANGE_EPOCH") == 1))))
               ;
             else
               QuitCDF ("12.0", status, newVarName);
@@ -634,9 +656,9 @@ switch (argC) {
                         (icount+1));
     }
 
-    /*********************************************/
-    /* Load the Variable Attributes.             */
-    /*********************************************/
+    /*********************************************
+    * Load the Variable Attributes.             
+    *********************************************/
 
     if (numvAttrs > 0) {
 
@@ -658,7 +680,7 @@ switch (argC) {
         if (status < CDF_OK) QuitCDF ("16.0", status, attrName);
  
         if (augmentlabel) {
-          if ((strcmpIgCase (attrName, "augment_labl") != 0)) 
+          if ((strcmpIgCase (attrName, "augment_labl") == 1)) 
             augmentlabelExist = TRUE;
         }
           
@@ -700,10 +722,10 @@ switch (argC) {
               if (STRINGdataType(dataTypeX)) 
                 *(((char *)value)+numElemsX) = NUL;
 /*
-              if ((strcmpIgCase (varName, "epoch") != 0) ||
-                  (strcmpIgCase (varName, "range_epoch") != 0)) {
+              if ((strcmpIgCase (varName, "epoch") == 1) ||
+                  (strcmpIgCase (varName, "range_epoch") == 1)) {
 */
-              if (strcmpIgCase (varName, "range_epoch") != 0) {
+              if (strcmpIgCase (varName, "range_epoch") == 1) {
 	          strcpyX (newVarName, varName, 0);
 	      } else {
                 if (usePrefix && !dataOnly)
@@ -718,10 +740,10 @@ switch (argC) {
                                GET_, zENTRY_DATATYPE_, &dataTypeY,
                                      zENTRY_NUMELEMS_, &numElemsY,
                                NULL_);
-              /****************************************************************
-              * If already exists, do thing.
-              * Otherwise, write it.
-              ****************************************************************/
+              /***************************************************************
+              * If already exists, do nothing.                              
+              * Otherwise, write it.                                       
+              ***************************************************************/
               if (status == CDF_OK) {
                 if (!usePrefix) {
                   cdf_FreeMemory (value, FatalError);
@@ -737,7 +759,7 @@ switch (argC) {
                 * to reflect the prefixed variable name in the merged file.
                 **************************************************************/
                 if (usePrefix && !dataOnly) {
-                  if (strcmpIgCase (attrName, dspType) != 0) {
+                  if (strcmpIgCase (attrName, dspType) == 1) {
                     changed = FALSE;
                     newValue = (char *) cdf_AllocateMemory (strlen(value)+150,
                                                             FatalError);
@@ -768,7 +790,7 @@ switch (argC) {
                   } else {
                     ifound = FALSE;
                     for (ii = 0; ii < sp2vAttrs; ii++) {
-                      if (strcmpIgCase (attrName, special2vAttrs[ii]) != 0) {
+                      if (strcmpIgCase (attrName, special2vAttrs[ii]) == 1) {
                         ifound = TRUE;
                         break;
                       }
@@ -796,10 +818,10 @@ switch (argC) {
                                    NULL_);
                   if (status < CDF_OK) {
 /*
-                    if ((strcmpIgCase (varName, "epoch") == 0) &&
-                        (strcmpIgCase (varName, "range_epoch") == 0)) 
+                    if ((strcmpIgCase (varName, "epoch") == 1) &&
+                        (strcmpIgCase (varName, "range_epoch") == 1)) 
 */
-                    if (strcmpIgCase (varName, "range_epoch") == 0) 
+                    if (strcmpIgCase (varName, "range_epoch") == 1) 
                       QuitCDF ("19.7", status, newVarName);
                   }
                 }
@@ -830,9 +852,9 @@ switch (argC) {
 
   }
 
-  /*********************************************/
-  /* Add the AUGMENT_LABL attribute if needed. */
-  /*********************************************/
+  /*********************************************
+  * Add the AUGMENT_LABL attribute if needed. 
+  *********************************************/
   if (augmentlabel) {
     if (!augmentlabelExist) {
       status = CDFlib (SELECT_, CDF_, ido,
@@ -863,9 +885,9 @@ switch (argC) {
     }
   }
 
-  /****************************************/
-  /* Load the Variable data.              */
-  /****************************************/
+  /**************************************** 
+  * Load the Variable data.               
+  ****************************************/
 
   for (icount = 0; icount < numCDFs; icount++) {
 
@@ -874,6 +896,7 @@ switch (argC) {
 
     status = CDFlib (OPEN_, CDF_, CDFpaths[icount], &id,
                      SELECT_, CDF_zMODE_, zMODEon2,
+                              CDF_READONLY_MODE_, READONLYon,
                      GET_, CDF_MAJORITY_, &major,
                            CDF_NUMzVARS_, &numzVars,
                      NULL_);   
@@ -903,10 +926,10 @@ switch (argC) {
       * Acquire the min/max epoch time for RANGE_EPOCH variable, only.
       ************************************************************************/
 /*
-      if ((strcmpIgCase (varName, "epoch") != 0) ||
-          (strcmpIgCase (varName, "range_epoch") != 0)) {
+      if ((strcmpIgCase (varName, "epoch") == 1) ||
+          (strcmpIgCase (varName, "range_epoch") == 1)) {
 */
-      if (strcmpIgCase (varName, "range_epoch") != 0) {
+      if (strcmpIgCase (varName, "range_epoch") == 1) {
         
         status = CDFlib (SELECT_, zVAR_RECNUMBER_, 0L,
                                   zVAR_RECCOUNT_, 2L,
@@ -1239,10 +1262,11 @@ char *outName;
   char temp[CDF_VAR_NAME_LEN256+1];
   MakeNUL (temp);
   if (numTokens != 0) 
-    sprintf (temp, "%s_", prefixes[index]);
+    snprintf (temp, (size_t) sizeof(temp), "%s_", prefixes[index]);
   else
-    sprintf (temp, "file%d_", (index+1)); 
-  sprintf (EofS(temp), "%s", inName);
+    snprintf (temp, (size_t) sizeof(temp), "file%d_", (index+1)); 
+  snprintf (EofS(temp), (size_t) sizeof(temp)-strlen(temp), "%s", 
+	    inName);
   strcpyX (outName, temp, 0);
 }
 
@@ -1368,8 +1392,8 @@ char *msg;
 void ConstructOutputFileName (name)
 char *name;
 { 
-  char fileName[CDF_PATHNAME_LEN+1];
-  char fileNaming[CDF_PATHNAME_LEN+1];
+  char fileName[DU_MAX_PATH_LEN+1];
+  char fileNaming[DU_MAX_PATH_LEN+1];
   char dataVersion[21];
   char **fields;
   char *ptr;
@@ -1400,7 +1424,7 @@ char *name;
 
   found = FALSE;
   for (ix = 0; ix < numgAttrs; ix++) {
-    if (strcmpIgCase (attrs[ix], "file_naming_convention") != 0) {
+    if (strcmpIgCase (attrs[ix], "file_naming_convention") == 1) {
       found = TRUE;
       iy = ix;
       break;
@@ -1509,8 +1533,8 @@ char *name;
 void ConstructOutputFileName2 (name)
 char *name;
 { 
-  char fileName[CDF_PATHNAME_LEN+1];
-  char fileNaming[CDF_PATHNAME_LEN+1];
+  char fileName[DU_MAX_PATH_LEN+1];
+  char fileNaming[DU_MAX_PATH_LEN+1];
   char dataVersion[21];
   char **fields;
   char *ptr;
@@ -1524,7 +1548,7 @@ char *name;
   
   found = FALSE;
   for (ix = 0; ix < lineAttrs; ix++) {
-    if (strcmpIgCase (attrs[ix], "file_naming_convention") != 0) {
+    if (strcmpIgCase (attrs[ix], "file_naming_convention") == 1) {
       found = TRUE;
       iy = ix;
       break;
@@ -1596,7 +1620,7 @@ char *name;
 
       found = FALSE;
       for (ix = 0; ix < lineAttrs; ix++) {
-        if (strcmpIgCase (attrs[ix], "data_version") != 0) {
+        if (strcmpIgCase (attrs[ix], "data_version") == 1) {
           found = TRUE;
           iy = ix;
           break;
@@ -1626,7 +1650,7 @@ char *name;
 void GetOutputFileName (name)
 char *name;
 { 
-  char fileName[CDF_PATHNAME_LEN+1];
+  char fileName[DU_MAX_PATH_LEN+1];
   char sourceName[21], dataType[21], descriptor[21], dataVersion[21];
   long srcLen, typLen, dspLen, verLen;
   char timeStamp[9];
@@ -1722,7 +1746,7 @@ char *name;
 void GetOutputFileName2 (name)
 char *name;
 { 
-  char fileName[CDF_PATHNAME_LEN+1];
+  char fileName[DU_MAX_PATH_LEN+1];
   char timeStamp[9];
   int ix, iy;
   Logical found;
@@ -1733,7 +1757,7 @@ char *name;
   MakeNUL (name);
 
   for (ix = 0; ix < lineAttrs; ix++) {
-    if (strcmpIgCase (attrs[ix], "Logical_file_id") != 0) {
+    if (strcmpIgCase (attrs[ix], "Logical_file_id") == 1) {
       found = TRUE;
       iy = ix;
       break;
@@ -1747,7 +1771,7 @@ char *name;
 
   found = FALSE;
   for (ix = 0; ix < lineAttrs; ix++) {
-    if (strcmpIgCase (attrs[ix], "Source_name") != 0) {
+    if (strcmpIgCase (attrs[ix], "Source_name") == 1) {
       found = TRUE;
       iy = ix; 
       break;
@@ -1765,7 +1789,7 @@ char *name;
 
   found = FALSE;
   for (ix = 0; ix < lineAttrs; ix++) {
-    if (strcmpIgCase (attrs[ix], "Descriptor") != 0) {
+    if (strcmpIgCase (attrs[ix], "Descriptor") == 1) {
       found = TRUE;
       iy = ix;
       break;
@@ -1785,7 +1809,7 @@ char *name;
 
   found = FALSE;
   for (ix = 0; ix < lineAttrs; ix++) {
-    if (strcmpIgCase (attrs[ix], "Data_type") != 0) {
+    if (strcmpIgCase (attrs[ix], "Data_type") == 1) {
       found = TRUE;
       iy = ix;
       break;
@@ -1810,7 +1834,7 @@ char *name;
 
   found = FALSE;
   for (ix = 0; ix < lineAttrs; ix++) {
-    if (strcmpIgCase (attrs[ix], "data_version") != 0) {
+    if (strcmpIgCase (attrs[ix], "data_version") == 1) {
       found = TRUE;
       iy = ix;
       break;
@@ -1854,6 +1878,7 @@ char *timeStamp;
     if (status != CDF_OK) QuitCDF ("GetDataTimeStamp", status, CDFpaths[ix]);
 
     status =  CDFlib (SELECT_, CDF_, idi,
+                               CDF_READONLY_MODE_, READONLYon,
                       GET_, CDF_NUMrVARS_, &numrVars,
                             CDF_NUMzVARS_, &numzVars,
                       NULL_);
@@ -1884,10 +1909,10 @@ char *timeStamp;
     found = FALSE;
     for (iz = 0; iz < (int) numVars; iz++) {
 /*
-      if ((strcmpIgCase (varNames[iz], "epoch") != 0) ||
-          (strcmpIgCase (varNames[iz], "range_epoch") != 0)) {
+      if ((strcmpIgCase (varNames[iz], "epoch") == 1) ||
+          (strcmpIgCase (varNames[iz], "range_epoch") == 1)) {
 */
-      if (strcmpIgCase (varNames[iz], "range_epoch") != 0) {
+      if (strcmpIgCase (varNames[iz], "range_epoch") == 1) {
         iy = iz;
         found = TRUE;
         break;
@@ -1974,27 +1999,27 @@ CDFid ids;
                      NULL_);
     if (status < CDF_OK) QuitCDF ("5.0", status, attrName);
 
-    if (strcmpIgCase (attrName, "File_naming_convention") != 0 &&
+    if (strcmpIgCase (attrName, "File_naming_convention") == 1 &&
         numgEntries == 0)
       fnc = TRUE;
-    if (strcmpIgCase (attrName, "Logical_file_id") != 0 && 
+    if (strcmpIgCase (attrName, "Logical_file_id") == 1 && 
         numgEntries == 0)
       lfi = TRUE;
-    if (strcmpIgCase (attrName, "Logical_source") != 0 &&
+    if (strcmpIgCase (attrName, "Logical_source") == 1 &&
         numgEntries == 0)
       lsi = TRUE;
 
     if (dataOnly) 
       strcpyX (newAttrName, attrName, 0);
     else {
-      /*****************************************************/
-      /* Check for special attributes. Its                 */
-      /* ISTP-standard name can't be changed.              */
-      /*****************************************************/
+      /****************************************************
+      * Check for special attributes. Its                 
+      * ISTP-standard name can't be changed.             
+      ****************************************************/
       ifound = FALSE;
       if (usePrefix) {
         for (j = 0; j < sp1gAttrs; j++) {
-          if (strcmpIgCase (attrName, special1gAttrs[j]) != 0) {
+          if (strcmpIgCase (attrName, special1gAttrs[j]) == 1) {
             ifound = TRUE;
             break;
           }
@@ -2043,7 +2068,7 @@ CDFid ids;
           ifound = FALSE;
 /*          if (!master) {  */
             for (ir = 0; ir < sp2gAttrs; ir++) {
-              if (strcmpIgCase (newAttrName, special2gAttrs[ir]) != 0) {
+              if (strcmpIgCase (newAttrName, special2gAttrs[ir]) == 1) {
                 ifound = TRUE;
                 break;
               }
@@ -2059,7 +2084,7 @@ CDFid ids;
             else {
               ifound = FALSE;
               for (ir = 0; ir < sp3gAttrs; ir++) {
-                if (strcmpIgCase (newAttrName, special3gAttrs[ir]) != 0) {
+                if (strcmpIgCase (newAttrName, special3gAttrs[ir]) == 1) {
                   ifound = TRUE;
                   break;
                 }
@@ -2104,7 +2129,7 @@ CDFid ids;
                              NULL_);
             if (status < CDF_OK) QuitCDF ("7.5", status, newAttrName);
 
-            if (strcmpIgCase (newAttrName, "Logical_source") != 0) {
+            if (strcmpIgCase (newAttrName, "Logical_source") == 1) {
               int aa, bb;
               aa = 0;
               for (bb = (int) strlen (outputCDF)-1; bb >= 0; bb--) {
@@ -2123,7 +2148,7 @@ CDFid ids;
               status = CDFlib (PUT_, gENTRY_DATA_, CDF_CHAR, numE,                 
                                      outputCDF,
                                NULL_);
-            } else if (strcmpIgCase (newAttrName, "Logical_file_id") != 0) {
+            } else if (strcmpIgCase (newAttrName, "Logical_file_id") == 1) {
               if (EndsWithIgCase (outputCDF, ".cdf"))
                 numE = (long) strlen (outputCDF) - 4;
               else
@@ -2216,13 +2241,13 @@ void LoadgAttributes2 ()
   Logical iFound, fnc = FALSE, lfi = FALSE, lsi = FALSE;
 
   for (i = 0; i < lineAttrs; i++) {
-    if (strcmpIgCase (attrs[i], "File_naming_convention") != 0) {
+    if (strcmpIgCase (attrs[i], "File_naming_convention") == 1) {
       if (inentries[i] == -1)
         fnc = TRUE;
-    } else if (strcmpIgCase (attrs[i], "Logical_file_id") != 0) {
+    } else if (strcmpIgCase (attrs[i], "Logical_file_id") == 1) {
       if (inentries[i] == -1)
         lfi = TRUE;
-    } else if (strcmpIgCase (attrs[i], "Logical_source") != 0) {
+    } else if (strcmpIgCase (attrs[i], "Logical_source") == 1) {
       if (inentries[i] == -1)                                 
         lsi = TRUE;                                           
     }
@@ -2238,7 +2263,7 @@ void LoadgAttributes2 ()
     for (l = 0; l < ii; l++) {
       ifound = FALSE;
       for (ir = 0; ir < sp2gAttrs; ir++) {
-        if (strcmpIgCase (attrs[i], special2gAttrs[ir]) != 0) {
+        if (strcmpIgCase (attrs[i], special2gAttrs[ir]) == 1) {
           ifound = TRUE;
           break;
         }
@@ -2257,7 +2282,7 @@ void LoadgAttributes2 ()
                                   gENTRY_, (long) l,
                          NULL_);
 
-        if (strcmpIgCase (attrs[i], "Logical_source") != 0) {
+        if (strcmpIgCase (attrs[i], "Logical_source") == 1) {
           int aa, bb;
           aa = 0;
           for (bb = (int) strlen (outputCDF)-1; bb >= 0; bb--) {
@@ -2270,7 +2295,7 @@ void LoadgAttributes2 ()
           status = CDFlib (PUT_, gENTRY_DATA_, CDF_CHAR, numE,                 
                                  outputCDF,
                            NULL_);
-        } else if (strcmpIgCase (attrs[i], "Logical_file_id") != 0) {
+        } else if (strcmpIgCase (attrs[i], "Logical_file_id") == 1) {
           if (EndsWithIgCase (outputCDF, ".cdf"))
             numE = (long) strlen (outputCDF) - 4;
           else
@@ -2376,7 +2401,7 @@ char *masterFile;
 {
   FILE *masterfp;
   int  i, maxAttr, maxEntry;
-  char line[MAX_MERGELINE_LEN+1+1], item[MAX_MERGELINE_LEN+1];
+  char lineX[DU_MAX_PATH_LEN+1+1], item[DU_MAX_PATH_LEN+1];
   char *ptr, *ptr2;
   char **tmp;
   int  *tmp2;
@@ -2391,15 +2416,15 @@ char *masterFile;
   inentries = (int *) cdf_AllocateMemory (sizeof(int)*maxAttr, FatalError);
   entries = (char **) cdf_AllocateMemory (sizeof(char *)*maxEntry, FatalError);
   ib = FALSE;
-  while (fgets(line,MAX_MERGELINE_LEN,masterfp) != NULL) {
-    if (sscanf(line, "%s\n", item) != 1) continue;
-    for (i = 0; line[i] != NUL; i++)
-       if (line[i] == Nl) line[i] = '\0'; 
-    RemoveLeadingBlanks (line);
-    RemoveTrailingBlanks (line);
-    ptr = strchr (line, ':');
-    ptr2 = strchr (line, ' ');
-    if (ptr != NULL && ptr2 == NULL && line[(int)strlen(line)-1] == ':') { 
+  while (fgets(lineX,DU_MAX_PATH_LEN+1,masterfp) != NULL) {
+    if (sscanf(lineX, "%s\n", item) != 1) continue;
+    for (i = 0; lineX[i] != NUL; i++)
+       if (lineX[i] == Nl) lineX[i] = '\0'; 
+    RemoveLeadingBlanks (lineX);
+    RemoveTrailingBlanks (lineX);
+    ptr = strchr (lineX, ':');
+    ptr2 = strchr (lineX, ' ');
+    if (ptr != NULL && ptr2 == NULL && lineX[(int)strlen(lineX)-1] == ':') { 
       if (ib == TRUE) {
       /* No entry entered for the previous attribute... */
         inentries[lineAttrs-1] = -1;
@@ -2423,8 +2448,8 @@ char *masterFile;
         inentries = tmp2;
         maxAttr = maxAttr + 50;
       }
-      attrs[lineAttrs] = (char *) cdf_AllocateMemory (strlen(line)+1, FatalError);
-      strcpyX (attrs[lineAttrs], line, strlen(line)-1);
+      attrs[lineAttrs] = (char *) cdf_AllocateMemory (strlen(lineX)+1, FatalError);
+      strcpyX (attrs[lineAttrs], lineX, strlen(lineX)-1);
       lineAttrs++;
       ib = TRUE;
     } else {
@@ -2440,8 +2465,8 @@ char *masterFile;
         entries = tmp;
         maxEntry = maxEntry + 50;
       }
-      entries[lineEntries] = (char *) cdf_AllocateMemory (strlen(line)+1, FatalError);
-      strcpyX (entries[lineEntries], line, 0);
+      entries[lineEntries] = (char *) cdf_AllocateMemory (strlen(lineX)+1, FatalError);
+      strcpyX (entries[lineEntries], lineX, 0);
       if (ib) {
         inentries[lineAttrs-1] = lineEntries;
         ib = FALSE;
