@@ -28,6 +28,7 @@
 *   V1.5e 26-Jue-01, M Liu      Handle -delete option properly if no existing 
 *                               CDF.
 *   V2.0  19-May-05, M Liu      Allow space(s) in the CDF file name path.
+*   V3.3  04-Apr-11, M Liu      Modified to hanlde TT2000 epoch.
 ******************************************************************************/
 
 #include "skt2cdf.h"
@@ -147,6 +148,7 @@ char *argV[];
     TRUE, FALSE, FALSE, TRUE, 0
   };
   static char *reportTokens[] = { "errors", "warnings", "informationals" };
+
   /****************************************************************************
   * Parse qualifiers/options/parameters.
   ****************************************************************************/
@@ -183,9 +185,14 @@ char *argV[];
       /************************************************************************
       * Check for CDF pathname qualifier.
       ************************************************************************/
-      if (qop->qualEntered[CDFPATHqual])
+      if (qop->qualEntered[CDFPATHqual]) {
+	if (!ValidateQual(qop->qualOpt[CDFPATHqual], validQuals))
 		strcpyX (CDFpath, qop->qualOpt[CDFPATHqual], DU_MAX_PATH_LEN);
-      else
+	else {
+		DisplayError ("Invalid \"cdf\" qualifier.");
+		qopError = TRUE;
+	}
+       } else
 		CDFpath[0] = NUL;
       /************************************************************************
       * Check for `report' qualifier.  If absent, use defaults.
@@ -264,10 +271,23 @@ char *argV[];
   inString = FALSE;
   /****************************************************************************
   * Open skeleton table. Try several combinations for the skeleton table path.
+  * If the table file contains the ".skt" or ".SKT" extension, then uses it as
+  * is. Otherwise, appends ".skt", ".skeleton_table", ".SKT", ".skt;1" and
+  * ".SKT;1" to the table file in sequence to see if it exists. The last try
+  * is to use the table file name without extension after all preceeding
+  * combinations fail.
   ****************************************************************************/
   ExpandPath (SKTpath, SKTpathX);
-  SKTfp = fopen (SKTpathX, "r");
-  if (SKTfp == NULL) {
+  if (EndsWithIgCase(SKTpathX, ".skt") ||
+      EndsWithIgCase(SKTpathX, ".skt;1")) {
+    SKTfp = fopen (SKTpathX, "r");
+    if (SKTfp == NULL) {
+      WriteOut (stdout, "Unable to open skeleton table \"");
+      WriteOut (stdout, SKTpath);
+      WriteOut (stdout, "\".\n");
+      return FALSE;
+    }  
+  } else {
     strcatX (SKTpathX, ".skt", DU_MAX_PATH_LEN);
     SKTfp = fopen (SKTpathX, "r");
     if (SKTfp == NULL) {
@@ -287,16 +307,23 @@ char *argV[];
 	    strcatX (SKTpathX, ".SKT;1", DU_MAX_PATH_LEN);
 	    SKTfp = fopen (SKTpathX, "r");
 	    if (SKTfp == NULL) {
-	      WriteOut (stdout, "Unable to open skeleton table \"");
-	      WriteOut (stdout, SKTpath);
-	      WriteOut (stdout, "\".\n");
-	      return FALSE;
+	      ExpandPath (SKTpath, SKTpathX);
+	      SKTfp = fopen (SKTpathX, "r");
+	      if (SKTfp == NULL) {
+	        WriteOut (stdout, "Unable to open skeleton table \"");
+	        WriteOut (stdout, SKTpath);
+	        WriteOut (stdout, "\".\n");
+	        return FALSE;
+	      }
             }
 	  }
 	}
       }
     }
   }
+  WriteOut (stdout, "Making a cdf from skeleton table: \"");
+  WriteOut (stdout, SKTpathX);
+  WriteOut (stdout, "\"");
   /****************************************************************************
   * Open NRV file (may not exist).
   ****************************************************************************/
@@ -896,7 +923,11 @@ char *item;
 		if (status != NO_SUCH_CDF )
 		  if (!StatusHandlerS2C(status,delCDF)) return FALSE;
 	    }
-	
+            WriteOut (stdout, "  to  \"");
+	    WriteOut (stdout, CDFpath);
+	    if (!EndsWithIgCase(CDFpath, ".cdf"))
+	      WriteOut (stdout, ".cdf");
+	    WriteOut (stdout, "\"...\n");
 	    status = CDFlib (CREATE_, CDF_, CDFpath, rNumDims,
 					    rDimSizes, &id,
 			     PUT_, CDF_FORMAT_, format,
@@ -932,6 +963,10 @@ char *item;
 		if (status != NO_SUCH_CDF)
 		  if (!StatusHandlerS2C(status,delCDF)) return FALSE;
 	  }
+          WriteOut (stdout, CDFpath);
+          if (!EndsWithIgCase(CDFpath, ".cdf"))
+            WriteOut (stdout, ".cdf");
+          WriteOut (stdout, "\"...\n");
 	  status = CDFlib (CREATE_, CDF_, CDFpath, rNumDims,
 					  rDimSizes, &id,
 			   PUT_, CDF_FORMAT_, format,
@@ -1037,8 +1072,11 @@ char *item;
     case GaVALUEfield: {
       void *valuesB;
       long eNumElems;
+      int style;
       if (!STRINGdataType(eDataType)) {
-	if (!DecodeValues(item,eDataType,&eNumElems,&valuesB,EPOCH0_STYLE)) {
+        if (TT2000dataType(eDataType)) style = TT2000_3_STYLE;
+        else  style = EPOCH0_STYLE;
+	if (!DecodeValues(item,eDataType,&eNumElems,&valuesB,style)) {
 	  ParseError ("Illegal entry value(s).");
 	  return FALSE;
 	}
@@ -1233,8 +1271,8 @@ char *item;
     case VARRECVARYfield:
       if (strlen(item) > (size_t) 1) {
 	char msg[MAX_MSG_LEN+1];
-	sprintf (msg, "Extra character(s) after record variance (`%s').",
-		 &item[1]);
+	snprintf (msg, (size_t) sizeof(msg),
+		  "Extra character(s) after record variance (`%s').", &item[1]);
 	ParseError (msg);
 	return FALSE;
       }
@@ -1300,8 +1338,9 @@ char *item;
     case VARDIMVARYfield:
       if (strlen(item) > (size_t) 1) {
 	char msg[MAX_MSG_LEN+1];
-	sprintf (msg, "Extra character(s) after dimension variance (`%s').",
-		 &item[1]);
+	snprintf (msg, (size_t) sizeof(msg),
+		  "Extra character(s) after dimension variance (`%s').",
+		  &item[1]);
 	ParseError (msg);
 	return FALSE;
       }
@@ -1429,8 +1468,11 @@ char *item;
     case VaeVALUEfield: {
       void *eValueB;
       long eNumElems;
+      int style;
       if (!STRINGdataType(eDataType)) {
-	if (!DecodeValues(item,eDataType,&eNumElems,&eValueB,EPOCH0_STYLE)) {
+        if (TT2000dataType(eDataType)) style = TT2000_3_STYLE;
+        else style = EPOCH0_STYLE;
+	if (!DecodeValues(item,eDataType,&eNumElems,&eValueB,style)) {
 	  ParseError ("Illegal entry value(s).");
 	  return FALSE;
 	}
@@ -1576,8 +1618,11 @@ char *item;
     case VARVALUEfield: {
       void *value;
       long numElems;
+      int style;
       if (!STRINGdataType(vDataType)) {
-	if (!DecodeValues(item,vDataType,&numElems,&value,EPOCH0_STYLE)) {
+        if (TT2000dataType(vDataType)) style = TT2000_3_STYLE;
+        else style = EPOCH0_STYLE;
+	if (!DecodeValues(item,vDataType,&numElems,&value,style)) {
 	  ParseError ("Illegal variable value.\n");
 	  return FALSE;
 	}
@@ -1595,8 +1640,8 @@ char *item;
 	  WriteOut (stdout, "ERROR: Truncating value for variable `");
 	  WriteOut (stdout, varName);
 	  WriteOut (stdout, "' at line\n");
-	  sprintf (tempS, "       number %d (this is a non-fatal error).\n",
-		   lineN);
+	  snprintf (tempS, (size_t) sizeof(tempS),
+		    "       number %d (this is a non-fatal error).\n", lineN);
 	  WriteOut (stdout, tempS);
 	  value = item;
 	}
@@ -1608,7 +1653,8 @@ char *item;
 			"WARNING: Blank padding value for variable `");
 	      WriteOut (stdout, varName);
 	      WriteOut (stdout, "'\n");
-	      sprintf (tempS, "         at line number %d.\n", lineN);
+	      snprintf (tempS, (size_t) sizeof(tempS),
+			"         at line number %d.\n", lineN);
 	      WriteOut (stdout, tempS);
 	    }
 	    nBytes = (size_t) (vNumElems + 1);
@@ -1696,7 +1742,8 @@ char *expected;
 char *found;
 {
   char msg[MAX_MSG_LEN+1];
-  sprintf (msg, "Expecting `%s', found `%s' instead.", expected, found);
+  snprintf (msg, (size_t) sizeof(msg),
+	    "\nExpecting `%s', found `%s' instead.", expected, found);
   ParseError (msg);
   return;
 }
@@ -1711,8 +1758,9 @@ char *expected2;
 char *found;
 {
   char msg[MAX_MSG_LEN+1];
-  sprintf (msg, "Expecting `%s' or `%s', found `%s' instead.",
-	   expected1, expected2, found);
+  snprintf (msg, (size_t) sizeof(msg),
+	    "Expecting `%s' or `%s', found `%s' instead.",
+	    expected1, expected2, found);
   ParseError (msg);
   return;
 }
@@ -1728,8 +1776,9 @@ char *expected3;
 char *found;
 {
   char msg[MAX_MSG_LEN+1];
-  sprintf (msg, "Expecting `%s', `%s', or `%s', found `%s' instead.",
-	   expected1, expected2, expected3, found);
+  snprintf (msg, (size_t) sizeof(msg),
+	    "Expecting `%s', `%s', or `%s', found `%s' instead.",
+	    expected1, expected2, expected3, found);
   ParseError (msg);
   return;
 }
@@ -1744,8 +1793,9 @@ char *msg;
   char tempS[MAX_MSG_LEN+1];
   WriteOut (stdout, msg);
   WriteOut (stdout, "\n\n");
-  sprintf (tempS, "Error at (or before) line number %d which follows...\n\n",
-	   lineN);
+  snprintf (tempS, (size_t) sizeof(tempS),
+	    "Error at (or before) line number %d which follows...\n\n",
+	    lineN);
   WriteOut (stdout, tempS);
   WriteOut (stdout, line);
   WriteOut (stdout, "\n");      /* `line' has a NL at the end already. */
@@ -1877,9 +1927,9 @@ char *msg;
     CDFlib (SELECT_, CDF_STATUS_, status,
 	    GET_, STATUS_TEXT_, text,
 	    NULL_);
-    sprintf (tempS, "\n%s%s%s.\n\n%s\n", infoT,
-	     ((strlen(infoT) + strlen(msg) + 2 >
-	      (size_t) MAX_SCREENLINE_LEN) ? "\n" : " "), msg, text);
+    snprintf (tempS, (size_t) sizeof(tempS), "\n%s%s%s.\n\n%s\n", infoT,
+	      ((strlen(infoT) + strlen(msg) + 2 >
+	       (size_t) MAX_SCREENLINE_LEN) ? "\n" : " "), msg, text);
     WriteOut (stdout, tempS);
     return TRUE;
   }
@@ -1887,9 +1937,9 @@ char *msg;
     CDFlib (SELECT_, CDF_STATUS_, status,
 	    GET_, STATUS_TEXT_, text,
 	    NULL_);
-    sprintf (tempS, "\n%s%s%s.\n\n%s\n", warnT,
-	     ((strlen(warnT) + strlen(msg) + 2 >
-	      (size_t) MAX_SCREENLINE_LEN) ? "\n" : " "), msg, text);
+    snprintf (tempS, (size_t) sizeof(tempS), "\n%s%s%s.\n\n%s\n", warnT,
+	      ((strlen(warnT) + strlen(msg) + 2 >
+	       (size_t) MAX_SCREENLINE_LEN) ? "\n" : " "), msg, text);
     WriteOut (stdout, tempS);
     return TRUE;
   }
@@ -1898,9 +1948,9 @@ char *msg;
       CDFlib (SELECT_, CDF_STATUS_, status,
 	      GET_, STATUS_TEXT_, text,
 	      NULL_);
-      sprintf (tempS, "\n%s%s%s.\n\n%s\n", errorT,
-	       ((strlen(errorT) + strlen(msg) + 2 >
-		(size_t) MAX_SCREENLINE_LEN) ? "\n" : " "), msg, text);
+      snprintf (tempS, (size_t) sizeof(tempS), "\n%s%s%s.\n\n%s\n", errorT,
+	        ((strlen(errorT) + strlen(msg) + 2 >
+		 (size_t) MAX_SCREENLINE_LEN) ? "\n" : " "), msg, text);
       WriteOut (stdout, tempS);
     }
     ParseError ("");
